package de.duehl.algorithm.backtracking.eightqueens;

import java.util.ArrayList;
import java.util.List;

import de.duehl.algorithm.backtracking.AlgorithmState;
import de.duehl.basics.text.Text;

public class EightQueens implements AlgorithmState<EightQueensStep> {

    private final List<Square> queens;

    private final int dimension;

    private boolean chooseNextStepWasSuccessfull;

    public EightQueens(int dimension) {
        this.dimension = dimension;

        queens = new ArrayList<>();
    }

    @Override
    public boolean isVaild() {
        for (int firstIndex = 0; firstIndex < queens.size() - 1; ++firstIndex) {
            Square firstQueen = queens.get(firstIndex);
            for (int secondIndex = firstIndex + 1; secondIndex < queens.size(); ++secondIndex) {
                Square secondQueen = queens.get(secondIndex);
                if (queensWatchEachOther(firstQueen, secondQueen)) {
                    return false;
                }
            }
        }

        return true;
    }

    static boolean queensWatchEachOther(Square firstQueen, Square secondQueen) {
        int file1 = firstQueen.getFile();
        int file2 = secondQueen.getFile();
        int rank1 = firstQueen.getRank();
        int rank2 = secondQueen.getRank();

        if (file1 == file2) {
            return true;
        }
        else if (rank1 == rank2) {
            return true;
        }
        else {
            int fileDiff = Math.abs(file1 - file2);
            int rankDiff = Math.abs(rank1 - rank2);
            return fileDiff == rankDiff;
        }
    }

    @Override
    public boolean isComplete() {
        return queens.size() == dimension;
    }

    @Override
    public void forward(EightQueensStep step) {
        Square square = step.getSquare();
        queens.add(square);
        //System.out.println("forward:\n" + this);
    }

    @Override
    public void backward(EightQueensStep step) {
        Square square = step.getSquare();
        queens.remove(square);
        //System.out.println("backward:\n" + this);
    }

    @Override
    public EightQueensStep chooseNextStep(List<EightQueensStep> badSteps) {
        chooseNextStepWasSuccessfull = true;

        int firstEmptyFile = determineFirstEmptyFile();
        if (-1 == firstEmptyFile) {
            chooseNextStepWasSuccessfull = false;
            return null;
        }

        int firstEmptyRank = determineFirstEmptyRank();
        if (-1 == firstEmptyRank) {
            chooseNextStepWasSuccessfull = false;
            return null;
        }

        for (int file = firstEmptyFile; file < dimension; ++file) {
            for (int rank = firstEmptyRank; rank < dimension; ++rank) {
                EightQueensStep step = new EightQueensStep(new Square(file, rank));
                if (!badSteps.contains(step)) {
                    return step;
                }
            }
        }

        chooseNextStepWasSuccessfull = false;
        return null;
    }

    private int determineFirstEmptyFile() {
        for (int file = 0; file < dimension; ++file) {
            boolean fileFound = false;
            for (Square queen : queens) {
                if (queen.getFile() == file) {
                    fileFound = true;
                    break;
                }
            }
            if (!fileFound) {
                return file;
            }
        }

        return -1;
    }

    private int determineFirstEmptyRank() {
        for (int rank = 0; rank < dimension; ++rank) {
            boolean rankFound = false;
            for (Square queen : queens) {
                if (queen.getRank() == rank) {
                    rankFound = true;
                    break;
                }
            }
            if (!rankFound) {
                return rank;
            }
        }

        return -1;
    }

    /** Gibt an, ob bei der Auswahl des nächsten Schritts ein solcher erzeugt werden konnte. */
    @Override
    public boolean chooseNextStepWasSuccessfull() {
        return chooseNextStepWasSuccessfull;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        int squareWidth = 3;

        String frontSpace = Text.multipleString(" ", (squareWidth-1) / 2);
        String rearSpace = Text.multipleString(" ", squareWidth / 2);
        String queenString = frontSpace + "D" + rearSpace;
        String freeString = frontSpace + " " + rearSpace;

        String squareHorizontalSeparatingBar = "+" + Text.multipleString("-", squareWidth);
        String horizontalBar = "  " + Text.multipleString(squareHorizontalSeparatingBar, dimension)
                + "+";
        builder.append(horizontalBar).append("\n");
        for (int rank = dimension - 1; rank >= 0; --rank) {
            builder.append(rank + 1).append(" ");
            for (int file = 0; file < dimension; ++file) {
                builder.append("|");
                boolean qeenOnSquare = false;
                for (Square queen : queens) {
                    if (queen.getFile() == file && queen.getRank() == rank) {
                        qeenOnSquare = true;
                        break;
                    }
                }
                builder.append(qeenOnSquare ? queenString : freeString);
            }
            builder.append("|").append("\n");
            builder.append(horizontalBar).append("\n");
        }

        builder.append("   ");
        for (int file = 0; file < dimension; ++file) {
            String fileChar = "ABCDEFGHIJKLMNOPQRSTUVXYZ".substring(file, file + 1);
            builder.append(frontSpace).append(fileChar).append(rearSpace).append(" ");
        }
        builder.append("\n");


        return builder.toString();
    }

}
