package de.duehl.basics.io.textfile;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import de.duehl.basics.datetime.time.StopWatch;
import de.duehl.basics.io.Charset;
import de.duehl.basics.io.Reader;
import de.duehl.basics.text.NumberString;

/**
 * Diese Klasse ist die abstrakte Basis um eine Text-Datei einzulesen.
 *
 * @version 1.01     2019-06-18
 * @author Christian Dühl
 */

public abstract class FullTextFileReader {

    private static final int NUMBER_OF_LINES_READ_TO_PRINT_PROGRESS = 100_000;

    /** Name der einzulesenden Datei. */
    protected final String filename;

    /** Die Kodierung der einzulesenden Datei. */
    protected final Charset charset;

    /** Beschreibung des Zeileninhalts der einzulesenden Datei (für die Fortschrittsausgaben). */
    private String lineDescription;

    /** Zeitmessung. */
    private final StopWatch watch;

    /** Zählt die analysierten Zeilen. */
    private int countAnalysedLines;

    /** Anfang der Ausgaben für die Fortschrittsausgaben. */
    private String progressTextStart;

    /** Gibt an, ob leere Zeilen übersprungen werden sollen. Default ist false. */
    private boolean skipBlankLines;

    /** Gibt an, ob Fortschrittsausgaben nach STDOUT erfolgen sollen. Default: true. */
    private boolean verboose;

    /** Gibt an, ob die erste Zeile übersprungen werden soll. Default ist false. */
    private boolean skipFirstLine;

    /** Übersprungene erste Zeile, oder der leere String, falls keine übersprungen wird. */
    private String skippedFirstLine;

    /** Anzahl der Zeilen, nach der eine Fortschrittsausgabe nach STDOUT erfolgen soll. */
    private int numberOfLinesReadToPrintProgress;

    /** Liest die Datei ein. */
    private Reader reader;

    /** Gibt an, ob das Einlesen bereits vor Ende der Datei beendet werden soll. */
    private boolean stopReading;

    /**
     * Konstruktor.
     *
     * @param filename
     *            Name der einzulesenden Datei.
     * @param charset
     *            Die Kodierung der einzulesenden Datei.
     */
    public FullTextFileReader(String filename, Charset charset) {
        this.filename = filename;
        this.charset = charset;
        progressTextStart = "Einlesen";
        lineDescription = "";
        skipBlankLines = false;
        verboose = true;
        skipFirstLine = false;
        skippedFirstLine = "";
        numberOfLinesReadToPrintProgress = NUMBER_OF_LINES_READ_TO_PRINT_PROGRESS;
        stopReading = false;

        watch = new StopWatch();
    }

    /**
     * Setter für die Beschreibung des Zeileninhalts der einzulesenden Datei (für die
     * Fortschrittsausgaben).
     */
    public void setLineDescription(String lineDescription) {
        this.lineDescription = lineDescription;
    }

    /** Setter für den Anfang der Ausgaben für die Fortschrittsausgaben (Default ist "Einlesen"). */
    public void setProgressTextStart(String progressTextStart) {
        this.progressTextStart = progressTextStart;
    }

    /** Legt fest, dass leeren Zeilen übersprungen werden sollen. */
    public void skipFirstLine() {
        skipFirstLine = true;
    }

    /** Legt fest, dass keine leeren Zeilen übersprungen werden sollen. Dies ist der Default. */
    public void doNotSkipBlankLines() {
        skipBlankLines = false;
    }

    /** Legt fest, dass leeren Zeilen übersprungen werden sollen. */
    public void skipBlankLines() {
        skipBlankLines = true;
    }

    /** Legt fest, dass keine Fortschrittsausgaben nach STDOUT erfolgen sollen. */
    public void beQuiet() {
        verboose = false;
    }

    /**
     * Setter für die Anzahl der Zeilen, nach der eine Fortschrittsausgabe nach STDOUT erfolgen
     * soll (Default ist 100.000).
     */
    public void setNumberOfLinesReadToPrintProgress(int numberOfLinesReadToPrintProgress) {
        this.numberOfLinesReadToPrintProgress = numberOfLinesReadToPrintProgress;
    }

    /**
     * List die Datei ein.                                                                  <br><br>
     *
     * Der Aufruf kann in der folgenden Form erfolgen:              <br><tt>&nbsp;&nbsp;&nbsp;&nbsp;
     *     reader.read(line -> analyseLine(line));                                         </tt><br>
     * wobei die Methode                                            <br><tt>&nbsp;&nbsp;&nbsp;&nbsp;
     *     analyseLine                                                                     </tt><br>
     * eine Methode in der gleichen Klasse ist.
     *
     * @param analyzer
     *            Objekt, dass die gewünschten Zeilen der Datei verarbeitet.
     */
    public void read(NotEmptyLineAnalyzer analyzer) {
        watch.start();
        reader = openReader();

        if (skipFirstLine) {
            skippedFirstLine = reader.readNextLine();
        }

        countAnalysedLines = 0;
        String line;
        while ((line = reader.readNextLine()) != null && !stopReading) {
            if (!skipBlankLines || !line.trim().isEmpty()) {
                analyzer.analyseLine(line);
                ++countAnalysedLines;
                if (countAnalysedLines % numberOfLinesReadToPrintProgress == 0) {
                    printProgressOutputInLoop();
                }
            }
        }
        printProgressOutputAfterLoop();

        reader.close();
        watch.stop();
    }

    /** Öffnet den Reader zum Einlesen der Datei. */
    abstract protected Reader openReader();

    private void printProgressOutputInLoop() {
        printProgressOutput("bislang");
    }

    private void printProgressOutputAfterLoop() {
        printProgressOutput("insgesamt");
    }

    private void printProgressOutput(String additional) {
        if (verboose) {
            System.out.println(progressTextStart + " von " + NumberString.taupu(countAnalysedLines)
                    + " Zeilen" + (lineDescription.isEmpty() ? "" : " mit " + lineDescription)
                    + " in " + additional + ": " + getTime());
        }
    }

    /** Liefert während der Bearbeitung der Zeilen die Zeilennummer. */
    public int getLineNumber() {
        return reader.getLineNumber();
    }

    /** Liefert die Laufzeit zurück. */
    public String getTime() {
        return watch.getTime();
    }

    /**
     * Getter für die Übersprungene erste Zeile, oder der leere String, falls keine übersprungen
     * wird.
     */
    public String getSkippedFirstLine() {
        return skippedFirstLine;
    }

    /** Beendet das Einlesen bereits vor Ende der Datei. */
    public void stopReading() {
        stopReading = true;
    }

}
