package de.duehl.basics.io.textfile;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * Diese Klasse liest eine Liste mit den Texten aus den Zeilen einer Textdatei ein.
 *
 * Die Zeilen werden getrimmt. Dabei werden leere Zeilen und solche, die mit dem Kommentarzeichen
 * (ggf., nach Leerraum) überlesen. Kommt in einer Datenzeile weiter hinten das Kommentarzeichen
 * vor, so wird ab dort abgeschnitten und getrimmt.
 *
 * @version 1.01     2019-03-21
 * @author Christian Dühl
 */

public class StringsFromTextFileReader {

    /** Der Reader, der zum Einlesen der Datei verwendet wird. */
    private final FullTextFileReader reader;

    /** Das Kommentarzeichen. */
    private final String commentChar;

    /** Liste mit den ausgelesenen Texten, die hier aufgebaut wird. */
    private List<String> strings;

    /** Gibt an, ob der Reader ruhig sein soll. */
    private boolean quiet;

    /**
     * Konstruktor.
     *
     * @param reader
     *            Der Reader, der zum Einlesen der Datei verwendet wird.
     * @param commentChar
     *            Das Kommentarzeichen.
     */
    public StringsFromTextFileReader(FullTextFileReader reader, String commentChar) {
        this.reader = reader;
        this.commentChar = commentChar;

        quiet = false;
    }

    /** Legt fest, dass der Reader ruhig sein soll. */
    public void beQuiet( ) {
        quiet = true;
    }

    /** Liest die Daten aus der Datei ein. */
    public List<String> read() {
        strings = new ArrayList<>();

        if (quiet) {
            reader.beQuiet();
        }
        reader.read(line -> analyseLine(line));

        return strings;
    }

    private void analyseLine(String line) {
        String lineOut = line.trim();

        if (lineOut.isEmpty() || lineOut.startsWith(commentChar)) {
            return;
        }

        int commentIndex = lineOut.indexOf(commentChar);
        if (commentIndex > -1) {
            lineOut = lineOut.substring(0, commentIndex);
            lineOut = lineOut.trim();
            lineOut = lineOut.toLowerCase(Locale.GERMAN);
        }

        strings.add(lineOut);
    }

}
