package de.duehl.math.graph.ged.graph.io.ged4;

import static de.duehl.math.graph.ged.graph.io.ged4.Ged4LoadAndSaveConstants.*;

import java.util.List;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.basics.text.handling.BooleanHandling;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.Edges;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.graph.vertex.Vertices;
import de.duehl.math.graph.ged.pixel.PixelPoint;

/**
 * Diese Klasse erzeugt eine Darstellung des Graphen in ged4-Format zum Speichern in einer Datei.
 *
 * @version 1.01     2019-06-17
 * @author Christian Dühl
 */

public class Ged4FormatSaver {

    /** Eckenmenge des Graphen. */
    private final Vertices vertices;

    /** Kantenmenge des Graphen. */
    private final Edges edges;

    /** Enthält weitere Informationen zum Graphen. */
    private MetaData meta;

    /** In dem Builder wird der zu sichernde String aufgebaut. */
    private StringBuilder builder;

    /**
     * Konstruktor.
     *
     * @param graph
     *            Der zu speichernde Graph.
     */
    public Ged4FormatSaver(Graph graph) {
        this.meta = graph.getMeta();
        this.vertices = graph.getVertices();
        this.edges = graph.getEdges();
    }

    /** Gibt den Graphen als String gespeichert im ged3-Format zurück. */
    public String createSaveString() {
        builder = new StringBuilder();

        appendMetaData();
        appendVertices();
        appendEdges();

        return builder.toString();
    }

    /** Metadaten hinzufügen. */
    private void appendMetaData() {
        List<Variable> metaVariables = createMetaVariables();
        String meta = Variable.variableListToSemicolonJoinedEquationList(metaVariables);
        builder.append(META_LINE_START + ";" + meta + "\n");
    }

    private List<Variable> createMetaVariables() {
        List<Variable> metaVariables = CollectionsHelper.buildListFrom(
                new Variable(META_DIRECTED, BooleanHandling.booleanToString(meta.isDirected())),
                new Variable(META_DEFAULT_VERTEX_COLOR, meta.getDefaultVertexColor().toColorString()),
                new Variable(META_DEFAULT_EDGE_COLOR, meta.getDefaultEdgeColor().toColorString()),
                new Variable(META_DEFAULT_RADIUS, meta.getDefaultRadius()),
                new Variable(META_BACKGROUND_COLOR, meta.getBackgroundColor().toColorString()),
                new Variable(META_SHOW_GRID, BooleanHandling.booleanToString(meta.isShowGrid())),
                new Variable(META_GRID_DISTANCE, meta.getGridDistance()),
                new Variable(META_SNAP_IN_GRID, BooleanHandling.booleanToString(meta.isSnapInGrid())),
                new Variable(META_GRID_COLOR, meta.getGridColor().toColorString()),
                new Variable(META_BACKGROUND_PICTURE, meta.getBackgroundPicture()),
                new Variable(META_DEFAULT_EDGE_WIDTH, meta.getDefaultEdgeWidth())
                );
        return metaVariables;
    }

    /** Ecken hinzufügen. */
    private void appendVertices() {
        for (Vertex vertex : vertices) {
            appendVertex(vertex);
        }
    }

    private void appendVertex(Vertex vertex) {
        List<Variable> vertexVariables = createVertexVariables(vertex);
        String vertexString = Variable.variableListToSemicolonJoinedEquationList(vertexVariables);
        builder.append(VERTEX_LINE_START + ";" + vertexString + "\n");
    }

    private List<Variable> createVertexVariables(Vertex vertex) {
        List<Variable> vertexVariables = CollectionsHelper.buildListFrom(
                new Variable(VERTEX_NUMBER, vertex.getIndex()),
                new Variable(VERTEX_X, vertex.getCoordinates().getX()),
                new Variable(VERTEX_Y, vertex.getCoordinates().getY())
                );
        if (!vertex.hasDefaultColor()) {
            vertexVariables.add(new Variable(VERTEX_COLOR, vertex.getColor().toColorString()));
        }

        if (!vertex.hasDefaultRadius()) {
            vertexVariables.add(new Variable(VERTEX_RADIUS, vertex.getRadius()));
        }

        if (vertex.hasLabel()) {
            String label = Ged4Helper.hideUsedCharactersInLabel(vertex.getLabel());
            vertexVariables.add(new Variable(VERTEX_LABEL, label));
        }

        if (!vertex.hasDefaultLabelDistance()) {
            PixelPoint labelDistance = vertex.getLabelDistance();
            vertexVariables.add(new Variable(VERTEX_LABEL_X_DISTANCE, labelDistance.getX()));
            vertexVariables.add(new Variable(VERTEX_LABEL_Y_DISTANCE, labelDistance.getY()));
        }

        if (!vertex.hasDefaultLabelColor()) {
            vertexVariables.add(new Variable(VERTEX_LABEL_COLOR,
                    vertex.getLabelColor().toColorString()));
        }

        if (vertex.hasWeight()) {
            vertexVariables.add(new Variable(VERTEX_WEIGHT, vertex.getWeight()));
        }

        if (!vertex.hasDefaultWeightDistance()) {
            PixelPoint weightDistance = vertex.getWeightDistance();
            vertexVariables.add(new Variable(VERTEX_WEIGHT_X_DISTANCE, weightDistance.getX()));
            vertexVariables.add(new Variable(VERTEX_WEIGHT_Y_DISTANCE, weightDistance.getY()));
        }

        if (!vertex.hasDefaultWeightColor()) {
            vertexVariables.add(new Variable(VERTEX_WEIGHT_COLOR,
                    vertex.getWeightColor().toColorString()));
        }

        return vertexVariables;
    }

    /** Kanten hinzufügen. */
    private void appendEdges() {
        for (Edge edge : edges) {
            appendEdges(edge);
        }
    }

    private void appendEdges(Edge edge) {
        List<Variable> edgeVariables = createEdgeVariables(edge);
        String vertexString = Variable.variableListToSemicolonJoinedEquationList(edgeVariables);
        builder.append(EDGE_LINE_START + ";" + vertexString + "\n");
    }

    private List<Variable> createEdgeVariables(Edge edge) {
        List<Variable> edgeVariables = CollectionsHelper.buildListFrom(
                new Variable(EDGE_START_VERTEX, edge.getStartVertexIndex()),
                new Variable(EDGE_END_VERTEX, edge.getTargetVertexIndex())
                );
        if (!edge.hasDefaultColor()) {
            edgeVariables.add(new Variable(EDGE_COLOR, edge.getColor().toColorString()));
        }

        if (!edge.hasDefaultStyle()) {
            edgeVariables.add(new Variable(EDGE_STYLE, edge.getStyle().toString()));
        }

        if (!edge.hasDefaultWidth()) {
            edgeVariables.add(new Variable(EDGE_WIDTH, edge.getWidth()));
        }

        if (edge.hasLabel()) {
            String label = Ged4Helper.hideUsedCharactersInLabel(edge.getLabel());
            edgeVariables.add(new Variable(EDGE_LABEL, label));
        }

        if (edge.hasDefaultLabelDistance()) {
            PixelPoint labelDistance = edge.getLabelDistance();
            edgeVariables.add(new Variable(EDGE_LABEL_X_DISTANCE, labelDistance.getX()));
            edgeVariables.add(new Variable(EDGE_LABEL_Y_DISTANCE, labelDistance.getY()));
        }

        if (!edge.hasDefaultLabelColor()) {
            edgeVariables.add(new Variable(EDGE_LABEL_COLOR, edge.getLabelColor().toColorString()));
        }

        if (!edge.hasDefaultWeight()) {
            edgeVariables.add(new Variable(EDGE_WEIGHT, edge.getWeight()));
        }

        if (edge.hasDefaultWeightDistance()) {
            PixelPoint weightDistance = edge.getWeightDistance();
            edgeVariables.add(new Variable(EDGE_WEIGHT_X_DISTANCE, weightDistance.getX()));
            edgeVariables.add(new Variable(EDGE_WEIGHT_Y_DISTANCE, weightDistance.getY()));
        }

        if (!edge.hasDefaultWeightColor()) {
            edgeVariables.add(new Variable(EDGE_WEIGHT_COLOR, edge.getWeightColor().toColorString()));
        }

        if (!edge.hasDefaultCapacity()) {
            edgeVariables.add(new Variable(EDGE_CAPACITY, edge.getCapacity()));
        }

        if (edge.hasDefaultCapacityDistance()) {
            PixelPoint capacityDistance = edge.getCapacityDistance();
            edgeVariables.add(new Variable(EDGE_CAPACITY_X_DISTANCE, capacityDistance.getX()));
            edgeVariables.add(new Variable(EDGE_CAPACITY_Y_DISTANCE, capacityDistance.getY()));
        }

        if (!edge.hasDefaultCapacityColor()) {
            edgeVariables.add(new Variable(EDGE_CAPACITY_COLOR, edge.getCapacityColor().toColorString()));
        }

        if (!edge.hasDefaultFlow()) {
            edgeVariables.add(new Variable(EDGE_FLOW, edge.getFlow()));
        }

        if (edge.hasDefaultFlowDistance()) {
            PixelPoint flowDistance = edge.getFlowDistance();
            edgeVariables.add(new Variable(EDGE_FLOW_X_DISTANCE, flowDistance.getX()));
            edgeVariables.add(new Variable(EDGE_FLOW_Y_DISTANCE, flowDistance.getY()));
        }

        if (!edge.hasDefaultFlowColor()) {
            edgeVariables.add(new Variable(EDGE_FLOW_COLOR, edge.getFlowColor().toColorString()));
        }


//        builder.append(EDGE_START_VERTEX + "=" + edge.getStartVertexIndex() + ";");
//        builder.append(EDGE_END_VERTEX + "=" + edge.getTargetVertexIndex() + ";");
//        builder.append(EDGE_COLOR + "=" + edge.getColor().toColorString() + ";");
//        builder.append(EDGE_STYLE + "=" + edge.getStyle() + ";");
//        builder.append(EDGE_WIDTH + "=" + edge.getWidth() + ";");
//        String label = Ged4Helper.hideUsedCharactersInLabel(edge.getLabel());
//        builder.append(EDGE_LABEL + "=" + label + ";");
//        builder.append(EDGE_LABEL_X_DISTANCE + "=" + edge.getLabelDistance().getX() + ";");
//        builder.append(EDGE_LABEL_Y_DISTANCE + "=" + edge.getLabelDistance().getY() + ";");
//        builder.append(EDGE_LABEL_COLOR + "=" + edge.getLabelColor().toColorString() + ";");
//        builder.append(EDGE_WEIGHT + "=" + edge.getWeight() + ";");
//        builder.append(EDGE_WEIGHT_X_DISTANCE + "=" + edge.getWeightDistance().getX() + ";");
//        builder.append(EDGE_WEIGHT_Y_DISTANCE + "=" + edge.getWeightDistance().getY() + ";");
//        builder.append(EDGE_WEIGHT_COLOR + "=" + edge.getWeightColor().toColorString() + ";");
//        builder.append(EDGE_CAPACITY + "=" + edge.getCapacity() + ";");
//        builder.append(EDGE_CAPACITY_X_DISTANCE + "=" + edge.getCapacityDistance().getX() + ";");
//        builder.append(EDGE_CAPACITY_Y_DISTANCE + "=" + edge.getCapacityDistance().getY() + ";");
//        builder.append(EDGE_CAPACITY_COLOR + "=" + edge.getCapacityColor().toColorString() + ";");
//        builder.append(EDGE_FLOW + "=" + edge.getFlow() + ";");
//        builder.append(EDGE_FLOW_X_DISTANCE + "=" + edge.getFlowDistance().getX() + ";");
//        builder.append(EDGE_FLOW_Y_DISTANCE + "=" + edge.getFlowDistance().getY() + ";");
//        builder.append(EDGE_FLOW_COLOR + "=" + edge.getFlowColor().toColorString());
        return edgeVariables;
    }

}
