package de.duehl.math.graph.ged.graph.subGraph;

import java.util.ArrayList;
import java.util.List;

import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

/**
 * Diese Klasse erstellt Teilgraphen mit einer Ecke und ihren Nachbarn.
 *
 * @version 1.02     2019-06-14
 * @author Christian Dühl
 */

public class VertexWithItsNeigboursSubGraphCreator extends SubgraphCreator {

    /** Die Startecke im originalen Graphen. */
    private Vertex startVertex;

    /**
     * Die Liste der Nachbarn der Startecke im originalen Graphen, zu denen Kanten von der
     * Startecke aus laufen.
     */
    private List<Vertex> originalForwardNeighbours;

    /**
     * Die Liste der Nachbarn der Startecke im originalen Graphen, von denen Kanten zur Startecke
     * laufen.
     */
    private List<Vertex> originalBackwardNeighbours;

    /** Kopie der Startecke im Teilgraphen. */
    private Vertex newStartVertex;

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSize
     *            Größe der quadratischen Teilgraphen.
     */
    public VertexWithItsNeigboursSubGraphCreator(Graph graph, int subGraphSize) {
        super(graph, subGraphSize, subGraphSize);
    }

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSizeX
     *            Breite der Teilgraphen.
     * @param subGraphSizeY
     *            Höhe der Teilgraphen.
     */
    public VertexWithItsNeigboursSubGraphCreator(Graph graph, int subGraphSizeX, int subGraphSizeY) {
        super(graph, subGraphSizeX, subGraphSizeY);
    }

    /**
     * Erzeugt einen Teilgraphen der eine Ecke mit ihren Nachbarn anzeigt.
     *
     * @param vertexIndex
     *            Index der Ecke.
     */
    public Graph createVertexWithItsNeigboursSubGraph(int vertexIndex) {
        return createVertexWithItsNeigboursSubGraph(graph.getVertex(vertexIndex));
    }

    /**
     * Erzeugt einen Teilgraphen der eine Ecke mit ihren Nachbarn anzeigt.
     *
     * @param vertexIndex
     *            Ecke.
     */
    public Graph createVertexWithItsNeigboursSubGraph(Vertex vertexIndex) {
        this.startVertex = vertexIndex;

        collectAllNeighbours();

        createSubGraphWithCopiedMeta();

        createCopyOfStartVertexForSubGraph();
        addVerticesToSubGraph();

        addNewEdgesToSubGraph();

        //maximizeSubGraph(subGraph);
        maximizeSubGraph(subGraph, newStartVertex);

        return subGraph;
    }

    /** Sammelt alle Nachbarn zu denen und von denen Kanten von der Startecke aus laufen. */
    private void collectAllNeighbours() {
        originalForwardNeighbours = graph.getForwardNeighboursAsVertices(startVertex);
        if (graph.isDirected()) {
            originalBackwardNeighbours = graph.getBackwardNeighboursAsVertices(startVertex);
        }
        else {
            originalBackwardNeighbours = new ArrayList<>();
            /*
             * Bei ungerichteten Graphen geben beide Methoden ALLE Nachbarn zurück. Also reicht
             * der Aufruf von einer.
             */
        }
    }

    private void createCopyOfStartVertexForSubGraph() {
        newStartVertex = new Vertex(startVertex);
    }

    /** Fügt Kopien der interessanten Ecken zum neuen Teilgraphen hinzu. */
    private void addVerticesToSubGraph() {
        subGraph.addVertex(newStartVertex); // Index 0 - wird automatisch korrigiert
        for (Vertex neighbourVertex : originalForwardNeighbours) {
            Vertex subGraphVertex = new Vertex(neighbourVertex);
            subGraph.addVertex(subGraphVertex); // Index 1 bis n - wird automatisch korrigiert.
        }
        for (Vertex neighbourVertex : originalBackwardNeighbours) {
            Vertex subGraphVertex = new Vertex(neighbourVertex);
            subGraph.addVertex(subGraphVertex); // Index 1 bis n - wird automatisch korrigiert.
        }
    }

    /** Neue Kanten erzeugen. */
    private void addNewEdgesToSubGraph() {
        for (int originalForwardNeighbourIndex = 0;
                originalForwardNeighbourIndex < originalForwardNeighbours.size();
                ++originalForwardNeighbourIndex) {
            addNewEdgeFromCopiedStartVertexToCopiedForwardNeighbour(originalForwardNeighbourIndex);
        }
        for (int originalBackwardNeighbourIndex = 0;
                originalBackwardNeighbourIndex < originalBackwardNeighbours.size();
                ++originalBackwardNeighbourIndex) {
            addNewEdgeFromCopiedBackwardNeighbourToCopiedStartVertex(originalBackwardNeighbourIndex);
        }
    }

    private void addNewEdgeFromCopiedStartVertexToCopiedForwardNeighbour(
            int originalForwardNeighbourIndex) {
        int indexOfCopiedStartVertex = 0;
        int indexOfCopiedForwardNeighbour = 1      // Startecke
                + originalForwardNeighbourIndex;   // bisherige Vorwärts-Nachbar
        subGraph.addEdge(indexOfCopiedStartVertex, indexOfCopiedForwardNeighbour);
    }

    private void addNewEdgeFromCopiedBackwardNeighbourToCopiedStartVertex(
            int originalBackwardNeighbourIndex) {
        int indexOfCopiedStartVertex = 0;
        int indexOfCopiedBackwardNeighbour = 1      // Startecke
                + originalForwardNeighbours.size()  // alle Vorwärts-Nachbar
                + originalBackwardNeighbourIndex;   // bisherige Rückwärts-Nachbar
        subGraph.addEdge(indexOfCopiedBackwardNeighbour, indexOfCopiedStartVertex);
    }

}
