package de.duehl.math.graph.ged.ui;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.Edges;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.resources.IconDefinitions;
import de.duehl.math.graph.ged.ui.creation.gui.CreateGui;
import de.duehl.math.graph.ged.ui.creation.gui.DirectedGraphInsertEdgeModusButtonPanelCreator;
import de.duehl.math.graph.ged.ui.creation.gui.GraphInsertEdgeModusButtonPanelCreator;
import de.duehl.math.graph.ged.ui.creation.gui.UndirectedGraphInsertEdgeModusButtonPanelCreator;
import de.duehl.math.graph.ged.ui.creation.menu.MenuCreation;
import de.duehl.math.graph.ged.ui.dialogs.EdgeDeleteDialog;
import de.duehl.math.graph.ged.ui.dialogs.EdgeDialog;
import de.duehl.math.graph.ged.ui.dialogs.GraphErrorDialog;
import de.duehl.math.graph.ged.ui.dialogs.ShowTextDialog;
import de.duehl.math.graph.ged.ui.dialogs.SizeChooser;
import de.duehl.math.graph.ged.ui.dialogs.VertexDialog;
import de.duehl.math.graph.ged.ui.graphpanel.GraphPanel;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.ErrorDialog;
import de.duehl.swing.ui.dialogs.base.NonModalFrameDialogBase;
import de.duehl.swing.ui.dialogs.html.ShowHTMLDialog;
import de.duehl.swing.ui.resources.IconLoader;
import de.duehl.basics.logging.Logger;
import de.duehl.basics.logic.Version;

/**
 * Diese Klasse verwaltet alles, was mit der grafischen Oberfläche zu tun hat.
 *
 * @version 1.01     2019-06-17
 * @author Christian Dühl
 */

public class GedGui extends NonModalFrameDialogBase implements Gui {

    private static final Image PROGRAM_IMAGE = loadProgramImage();

    /** Enthält die grafischen Elemente, die man für späteren Zugriff benötigt. */
    private GuiElements elements;

    /** Zugriff auf die Programmlogik */
    private final Logic logic;

    /** Soll-Breite des Graphenpanels. */
    private final int graphPanelWidth;

    /** Soll-Höhe des Graphenpanels. */
    private final int graphPanelHeight;

    /**
     * Konstruktor
     *
     * @param graphPanelWidth
     *            Soll-Breite des Graphenpanels.
     * @param graphPanelHeight
     *            Soll-Höhe des Graphenpanels.
     * @param logic
     *            Zugriff auf die Programmlogik
     */
    public GedGui(int graphPanelWidth, int graphPanelHeight, Logic logic) {
        super(PROGRAM_IMAGE, "DUMMY_TITLE");

        GuiTools.setNiceLayoutManager();

        this.graphPanelWidth = graphPanelWidth;
        this.graphPanelHeight = graphPanelHeight;
        this.logic = logic;

        fillDialog();
    }

    private static Image loadProgramImage() {
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        Image programImage = iconLoader.loadProgramIconImage();
        return programImage;
    }

    /** Erstellt die Oberfläche. */
    @Override
    protected void populateDialog() {
        elements = new GuiElements();
        elements.setProgramImage(PROGRAM_IMAGE);
        elements.setFrame(super.getFrame());

        createGui();
        createMenu();

        showFileName("");
    }

    private void createGui() {
        new CreateGui(graphPanelWidth, graphPanelHeight, elements, logic);
    }

    /** Erzeugt das Menü. */
    private void createMenu() {
        MenuCreation menue = new MenuCreation(logic, this, elements);
        menue.addTo(elements.getFrame());
    }

    /** Getter für den graphPanel */
    @Override
    public GraphPanel getGraphPanel() {
        GraphPanel graphPanel = elements.getGraphPanel();
        return graphPanel;
    }

    /** Setzt den Änderungsmarker. */
    @Override
    public void setChanged() {
        JLabel changeLabel = elements.getChangeLabel();
        changeLabel.setText("  *  ");
    }

    /**
     * Setzt den darzustellenden Graphen.
     *
     * @param graph
     *            darzustellender Graphen
     */
    @Override
    public void setGraph(Graph graph) {
        GraphPanel graphPanel = elements.getGraphPanel();
        graphPanel.setGraph(graph);
        graphPanel.validate();

        /* Checkboxen der Metadaten zum Gitter richtig setzen: */
        MetaData meta = graph.getMeta();
        elements.getShowGridCheckBox().setSelected(meta.isShowGrid());
        elements.getSnapInGridCheckBox().setSelected(meta.isSnapInGrid());
    }

    /** Gibt den dargestellten Graphen zurück. */
    @Override
    public Graph getGraph() {
        GraphPanel graphPanel = elements.getGraphPanel();
        return graphPanel.getGraph();
    }

    /**
     * Setzt den Statustext.
     *
     * @param statusText
     *            Anzuzeigender Text.
     */
    @Override
    public void setStatus(String statusText) {
        JLabel statusLabel = elements.getStatusLabel();
        statusLabel.setText(statusText);
        repaintGraphPanel();
    }

    /** Gibt den zugrundeliegenden JFrame zurück. */
    @Override
    public JFrame getFrame() {
        JFrame frame = elements.getFrame();
        return frame;
    }

    /** Unterbricht eine ggf. begonnene Bearbeitung. */
    @Override
    public void interruptEditing() {
        elements.getGraphPanel().interruptEditing();
    }

    /**
     * Lässt die Benutzung des Buttons und des Menüeintrags für den vorigen Stand aus der
     * Bearbeitungshistorie zu.
     */
    @Override
    public void enableUndo() {
        JButton previousButton = elements.getPreviousButton();
        JMenuItem previousMenuItem = elements.getPreviousHistoryMenuItem();
        previousButton.setEnabled(true);
        previousMenuItem.setEnabled(true);
    }

    /**
     * Verbietet die Benutzung des Buttons und des Menüeintrags für den vorigen Stand aus der
     * Bearbeitungshistorie.
     */
    @Override
    public void disableUndo() {
        JButton previousButton = elements.getPreviousButton();
        JMenuItem previousMenuItem = elements.getPreviousHistoryMenuItem();
        previousButton.setEnabled(false);
        previousMenuItem.setEnabled(false);
    }

    /**
     * Lässt die Benutzung des Buttons und des Menüeintrags für den nächsten Stand aus der
     * Bearbeitungshistorie zu.
     */
    @Override
    public void enableRedo() {
        JButton nextButton = elements.getNextButton();
        JMenuItem nextMenuItem = elements.getNextHistoryMenuItem();
        nextButton.setEnabled(true);
        nextMenuItem.setEnabled(true);
    }

    /**
     * Verbietet die Benutzung des Buttons und des Menüeintrags für den nächsten Stand aus der
     * Bearbeitungshistorie.
     */
    @Override
    public void disableRedo() {
        JButton nextButton = elements.getNextButton();
        JMenuItem nextMenuItem = elements.getNextHistoryMenuItem();
        nextButton.setEnabled(false);
        nextMenuItem.setEnabled(false);
    }

    /** Fertigt einen Snapshot vom graphenPanel an. */
    @Override
    public BufferedImage getSnapshot() {
        GraphPanel view = elements.getGraphPanel();
        int width = view.getWidth();
        int height = view.getHeight();

        BufferedImage image = new BufferedImage(width, height,
                BufferedImage.TYPE_INT_ARGB);

        synchronized (view.getTreeLock()) {
            view.paint(image.createGraphics());
        }

        return image;
    }

    // TODO Das Zeug in einen ErrorHanlder verfrachten und den in der überschriebene Methode
    // createErrorHanlder erzeugen!!!
    // Der muss von dem GlasPane... abgeleitet sein...

    /** Zeigt dem Benutzer einen Fehler an. */
    @Override
    public void error(String message) {
        error(message, null);
    }

    /** Zeigt dem Benutzer einen Fehler an. */
    @Override
    public void error(String message, Exception exception) {
        Graph graph = getGraph();
        Logger logger = logic.getLogger();
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        Image programImage = iconLoader.loadProgramIconImage();
        ErrorDialog dialog = new GraphErrorDialog(message, exception, graph,
                programImage, logger.getLogFileName(),
//                elements.getForegroundColorAsSwingColor(),
//                elements.getBackgroundColorAsSwingColor(),
                elements.getFrameLocation());
        dialog.setVisible(true);
        setStatus(message);
    }

    /** Wählt die Defaultfarbe für die Ecken aus. */
    @Override
    public GedColor chooseDefaultVertexColor() {
        GedColor oldDefault = getGraph().getMeta().getDefaultVertexColor();
        return chooseColor("Choose Default Vertex Color", oldDefault);
    }

    /**
     * Wählt eine Farbe aus.
     *
     * @param title
     *            Überschrift des Farbmenüs
     * @param oldDefault
     *            Alte Farbe
     * @return neue Farbe
     */
    private GedColor chooseColor(String title, GedColor oldDefault) {
        java.awt.Color swingColor = JColorChooser.showDialog(
                elements.getFrame(), title, oldDefault.toSwingColor());
        if (null == swingColor) {
            return oldDefault;
        }
        else {
            return new GedColor(swingColor);
        }
    }

    /** Wählt die Defaultfarbe für die Kanten aus. */
    @Override
    public GedColor chooseDefaultEdgeColor() {
        GedColor oldDefault = getGraph().getMeta().getDefaultEdgeColor();
        return chooseColor("Choose Default Edge Color", oldDefault);
    }

    /** Zeigt den Dialog zum Bearbeiten der Eckeneigenschaften an. */
    @Override
    public void showVertexPopupMenu(Vertex vertex, PixelPoint coords) {
        Graph graph = elements.getGraphPanel().getGraph();
        VertexDialog dialog = new VertexDialog(graph, vertex, coords, elements);
        dialog.setVisible(true);
        repaint();
    }

    /** Zeigt den Dialog zum Bearbeiten der Kanteneigenschaften an. */
    @Override
    public void showEdgePopupMenu(Edge edge, PixelPoint coords) {
        Graph graph = elements.getGraphPanel().getGraph();
        EdgeDialog dialog = new EdgeDialog(logic, graph, edge, coords, elements);
        dialog.setVisible(true);
        repaint();
    }

    /** Zeigt einen Textdialog mit dem übergebenen Titel und Inhalt an. */
    @Override
    public void showTextDialog(String title, String content) {
        new ShowTextDialog(title, content);
    }

    /**
     * Bestimmt aus einer Menge von Kanten, welche davon gelöscht werden sollen
     *
     * @param edgesAtCoords
     *            Menge mit den Kanten, die in der Nähe einer bestimmten Bildschirmkoordinate
     *            verlaufen.
     * @param coords
     *            Koordinaten
     * @return Menge der Kanten, die wirklich gelöscht werden sollen.
     */
    @Override
    public Edges determineEdgesToDelete(Edges edgesAtCoords, PixelPoint coords) {
        Graph graph = elements.getGraphPanel().getGraph();
        EdgeDeleteDialog dialog = new EdgeDeleteDialog(graph, edgesAtCoords, coords, elements);
        dialog.setVisible(true);
        return dialog.getEdgesToDelete();
    }

    /**
     * Zeigt den Dateinamen im Titel an.
     *
     * @param filename
     *            Dateiname
     */
    @Override
    public void showFileName(String filename) {
        String title = "GED - Graph Editor";
        if (filename != null && !filename.isEmpty()) {
            title = title + " - " + filename;
        }
        elements.getFrame().setTitle(title);
        refresh();
    }

    /**
     * Fragt den Benutzer nach dem Defaultradius für die Ecken.
     *
     * @param oldValue
     *            Alter Wert.
     */
    @Override
    public int askUserForSizeValue(int oldValue) {
        SizeChooser dialog = new SizeChooser(oldValue);
        int size = dialog.showDialog(new PixelPoint(100, 100));
        // TODO: aktuelle Koordinaten statt dessen!
        return size;
    }

    /** Zeichnet den Graphpanel neu. */
    @Override
    public void repaintGraphPanel() {
        GraphPanel graphPanel = getGraphPanel();
        graphPanel.repaint();
        //graphPanel.revalidate();
        //graphPanel.validate();
    }

    /** Zeigt die Größe des GraphPanels an. */
    @Override
    public void showGraphPanelSize(int width, int height) {
        JLabel label = elements.getSizeLabel();
        String size = " " + Integer.toString(width) + " : " + Integer.toString(height) + " ";
        label.setText(size);
    }

    /** Zeigt den Über-Dialog an. */
    @Override
    public void showAbout() {
        Version version = logic.getVersion();
        String versionNumber = version.getVersion();
        String versionDate   = version.getDate();
        String message = "GED - Graph Editor\n\n"
                + "Autor: Christian Dühl\n\n"
                + "Version: " + versionNumber + " vom " + versionDate + "\n\n";
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        Icon icon = iconLoader.loadIcon(IconDefinitions.GED_PROGRAMM_ICON);

        JOptionPane.showMessageDialog(getFrame(), message,
                "Über GED",
                JOptionPane.INFORMATION_MESSAGE,
                icon);
    }

    /** Zeigt die Hilfe an. */
    @Override
    public void showHelp() {
        showHTMLHelpDialog("Hilfe", "help/help.html");
    }

    /** Zeigt die offenen Punkte an. */
    @Override
    public void showTodo() {
        showHTMLHelpDialog("TODO", "todo.html");
    }

    /** Zeigt die Veränderungen an. */
    @Override
    public void showChanges() {
        showHTMLHelpDialog("Historie der Programmentwicklung", "changes.html");
    }

    private void showHTMLHelpDialog(String title, String filename) {
        ShowHTMLDialog dialog = new ShowHTMLDialog(title, PROGRAM_IMAGE,
                elements.getFrameLocation());
        URL url = this.getClass().getClassLoader().getResource(
                "de/duehl/math/graph/ged/documents/" + filename);
        dialog.showHtml(url);
        dialog.setVisible(true);
    }

    /**
     * Setzt die Größe der Canvas.
     *
     * @param width
     *            neue Breite
     * @param height
     *            neue Höhe
     */
    @Override
    public void setViewSize(int width, int height) {
        GraphPanel graphPanel = elements.getGraphPanel();
        //graphPanel.setSize(width, height);
        graphPanel.setPreferredSize(new Dimension(width, height));
        JFrame frame = elements.getFrame();
        frame.pack();
    }

    /** Fragt den Benutzer, ob das Programm beendet werden soll. */
    @Override
    public boolean askUserToQuit() {
        JFrame frame = elements.getFrame();
        return GuiTools.askUserToQuit(frame);
    }

    @Override
    public String askUserToOpenAsGed3() {
        return GuiTools.openFileWithExtension(getFrame(), ".ged3", logic.getDirectory());
    }

    @Override
    public String askUserToOpenAsGed4() {
        return GuiTools.openFileWithExtension(getFrame(), ".ged4", logic.getDirectory());
    }

    @Override
    public String askUserToSaveAsGed3() {
        FileFilter fileFilter = GuiTools.createExtensionFileFilter(".ged3");
        return GuiTools.saveFileAs(getFrame(), logic.getDirectory(), fileFilter);
    }

    @Override
    public String askUserToSaveAsGed4(String oldFilename) {
        FileFilter fileFilter = GuiTools.createExtensionFileFilter(".ged4");
        if (oldFilename.isEmpty()) {
            return GuiTools.saveFileAsWithTitle(
                    "Unter welchem Namen soll der Graph gespeichert werden?",
                    getFrame(), logic.getDirectory(), fileFilter);
        }
        else {
            return GuiTools.saveFileAsWithTitle(
                    "Unter welchem Namen soll der Graph gespeichert werden?",
                    getFrame(), logic.getDirectory(), fileFilter, oldFilename);
        }
    }

    @Override
    public String askUserAboutNameForScreenshot() {
        FileFilter fileFilter = GuiTools.createExtensionFileFilter(".png");
        return GuiTools.saveFileAs(getFrame(), logic.getDirectory(), fileFilter);
    }

    @Override
    public String askUserToOpenBackgroundPicture() {
        return GuiTools.openFileWithExtension(getFrame(), ".jpg", logic.getDirectory());
    }

    /** Schaltet den Graphen zwischen gerichtet und ungerichtet um. */
    @Override
    public void switchDirectedGraph(boolean directed) {
        GraphInsertEdgeModusButtonPanelCreator creator =
                directed ? new DirectedGraphInsertEdgeModusButtonPanelCreator(elements)
                         : new UndirectedGraphInsertEdgeModusButtonPanelCreator(elements);
        creator.create();

        elements.getFrame().revalidate();
    }

}
