package de.duehl.math.graph.ged.ui.creation.menu;

import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.math.graph.ged.ui.Gui;
import de.duehl.swing.ui.menu.MyMenuItem;
import de.duehl.swing.ui.menu.SingleMenuCreation;

/**
 * Diese Klasse stellt die Methoden zur Verfügung, um das Datei-Menü für die grafische Oberfläche
 * zu erstellen.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class FileMenuCreation implements SingleMenuCreation {

    static final boolean OPAQUE_SUBMENUES = false;

    /** Verbindung zur Programmlogik. */
    private final Logic logic;

    /** Verbindung zur Gui. */
    @SuppressWarnings("unused")
    private final Gui gui;

    /** Tastaturmaske für die Control-Taste. */
    private final int ctrlMask;

    /** Tastaturmaske für die Shift-Taste und die Control-Taste gleichzeitig. */
    @SuppressWarnings("unused")
    private final int shiftCtrlMask;

    /** Tastaturmaske für die Alt-Taste. */
    private final int altMask;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Verbindung zur Programmlogik.
     * @param gui
     *            Verbindung zur Gui.
     * @param ctrlMask
     *            Tastaturmaske für die Control-Taste.
     * @param shiftCtrlMask
     *            Tastaturmaske für die Shift-Taste und die Control-Taste gleichzeitig.
     */
    public FileMenuCreation(Logic logic, Gui gui, int ctrlMask, int shiftCtrlMask, int altMask) {
        this.logic = logic;
        this.gui = gui;
        this.ctrlMask = ctrlMask;
        this.shiftCtrlMask = shiftCtrlMask;
        this.altMask = altMask;
    }

    /** Erzeugt das Dateimenü. */
    @Override
    public JMenu createTheMenu() {
        JMenu menu = new JMenu("Datei");
        menu.setMnemonic('d');

        menu.add(createEmptyGraphMenuItem());
        menu.add(createLoadMenuItem());
        menu.add(createTheOpenFromOtherFormatsSubMenu());

        menu.addSeparator();

        menu.add(createSaveGed4FormatMenuItem());
        menu.add(createSaveAsGed4FormatMenuItem());
        menu.add(createTakeScreenshotMenuItem());
        menu.add(createTheSaveUnderOtherFormatsSubMenu());

        menu.addSeparator();

        menu.add(createExitMenuItem());

        return menu;
    }

    private JMenuItem createEmptyGraphMenuItem() {
        return new MyMenuItem("Neuer, leerer Graph")
                .mnemonic('n')
                .accelerator(KeyEvent.VK_N, ctrlMask)
                .actionListener(e -> logic.createEmptyFile());
    }

    private JMenuItem createLoadMenuItem() {
        return new MyMenuItem("Datei (.ged4) öffnen ...")
                .mnemonic('ö')
                .accelerator(KeyEvent.VK_O, ctrlMask)
                .actionListener(e -> logic.loadGed4File());
    }

    /** Untermenü für das Öffnen von Graphen in anderen Dateiformaten. */
    private JMenu createTheOpenFromOtherFormatsSubMenu() {
        JMenu menu = new JMenu("Graphendatei aus anderen Formaten öffnen");

        menu.setMnemonic('f');
        if (OPAQUE_SUBMENUES) {
            menu.setOpaque(true);
        }

        menu.add(createLoadFromGed1FormatMenuItem());
        menu.add(createLoadFromGed2FormatMenuItem());
        menu.add(createLoadFromGed3FormatMenuItem());
        menu.add(createLoadFromGed4FormatMenuItem());

        return menu;
    }

    private JMenuItem createLoadFromGed1FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged Format öffnen ...")
                .mnemonic('g')
                .actionListener(e -> logic.loadGedFile());
    }

    private JMenuItem createLoadFromGed2FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged2 Format öffnen ...")
                .mnemonic('2')
                .actionListener(e -> logic.loadGed2File());
    }

    private JMenuItem createLoadFromGed3FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged3 Format öffnen ...")
                .mnemonic('3')
                .actionListener(e -> logic.loadGed3File());
    }

    private JMenuItem createLoadFromGed4FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged4 Format öffnen ...")
                .mnemonic('4')
                .actionListener(e -> logic.loadGed4File());
    }

    private JMenuItem createSaveGed4FormatMenuItem() {
        return new MyMenuItem("Datei (.ged4) speichern")
                .mnemonic('s')
                .accelerator(KeyEvent.VK_S, ctrlMask)
                .actionListener(e -> logic.saveGed4File());
    }

    private JMenuItem createSaveAsGed4FormatMenuItem() {
        return new MyMenuItem("Datei (.ged4) speichern unter ...")
                .mnemonic('u')
                .accelerator(KeyEvent.VK_F12, 0)
                .actionListener(e -> logic.saveGed4FileAs());
    }

    private JMenuItem createTakeScreenshotMenuItem() {
        return new MyMenuItem("Aktuelle Ansicht als png-Bild speichern ...")
                .mnemonic('B')
                //.accelerator(KeyEvent.VK_Q, ctrlMask) // nicht mehr!
                .accelerator(KeyEvent.VK_O, ctrlMask)
                .actionListener(e -> logic.screenhot());
    }

    /** Untermenü für das Speichern in anderen Dateiformaten. */
    private JMenu createTheSaveUnderOtherFormatsSubMenu() {
        JMenu menu = new JMenu("Graphendatei in anderen Formaten speichern");

        menu.setMnemonic('p');
        if (OPAQUE_SUBMENUES) {
            menu.setOpaque(true);
        }

        menu.add(createSaveToGed1FormatMenuItem());
        menu.add(createSaveToGed2FormatMenuItem());
        menu.add(createSaveToGed3FormatMenuItem());
        menu.add(createSaveToGed4FormatMenuItem());

        return menu;
    }

    private JMenuItem createSaveToGed1FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged Format speichern ...")
                .mnemonic('g')
                .actionListener(e -> logic.saveGedFile());
    }

    private JMenuItem createSaveToGed2FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged2 Format speichern ...")
                .mnemonic('2')
                .actionListener(e -> logic.saveGed2File());
    }

    private JMenuItem createSaveToGed3FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged3 Format speichern ...")
                .mnemonic('3')
                .actionListener(e -> logic.saveGed3File());
    }

    private JMenuItem createSaveToGed4FormatMenuItem() {
        return new MyMenuItem("Graphendatei im .ged4 Format speichern ...")
                .mnemonic('4')
                .actionListener(e -> logic.saveGed4File());
    }

    private JMenuItem createExitMenuItem() {
        MyMenuItem menuItem = new MyMenuItem("Programm beenden")
                .mnemonic('b')
                .accelerator(KeyEvent.VK_F4, altMask)
                .actionListener(e -> logic.quitProgram());
        return menuItem;
    }

}
