package de.duehl.math.graph.ged.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JPanel;

import de.duehl.basics.text.NumberString;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.ui.GedGuiTools;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.math.graph.ged.ui.elements.EdgeStyleChooser;
import de.duehl.math.graph.ged.ui.elements.SubGraphButton;
import de.duehl.math.graph.ged.ui.elements.SwitchableValueDistanceAndColorChooser;
import de.duehl.math.graph.ged.ui.elements.ValueDistanceAndColorChooser;
import de.duehl.math.graph.ged.ui.elements.color.DefaultOrOwnColorChooser;
import de.duehl.math.graph.ged.ui.elements.radiusandwidth.RadiusOrWidthChooser;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.layout.FixedCenter;

/**
 * Diese Klasse stellt den Dialog für die Bearbeitung der Eigenschaften einer Kante dar.
 *
 * @version 1.01     2019-06-14
 * @author Christian Dühl
 */

public class EdgeDialog extends ModalDialogBase {

    private static final int SUB_GRAPH_SIZE = 150;

    private static final Color NOT_SHOWN_COLOR = Color.MAGENTA; // Wird nicht angezeigt

    /** Verbindung zur Logik. */
    private final Logic logic;

    /** Graph, der dargestellt wird. */
    private final Graph graph;

    /** Metadaten des Graphen, der dargestellt wird, zur Bequemlichkeit. */
    private final MetaData meta;

    /** Kante deren Daten der Dialog anzeigt und ändern lässt. */
    private final Edge originalEdge;

    /** Kopie der Kante deren Daten der Dialog anzeigt und ändern lässt. */
    private final Edge edge;

    /** Koordinaten, an denen der Dialog angezeigt werden soll. */
    private final PixelPoint coords;

    /** Enthält die grafischen Elemente. */
    private final GuiElements elements;

    private SubGraphButton subGraphButton;
    private RadiusOrWidthChooser widthChooser;
    private DefaultOrOwnColorChooser colorChooser;
    private ValueDistanceAndColorChooser labelChooser;
    private SwitchableValueDistanceAndColorChooser weightChooser;
    private SwitchableValueDistanceAndColorChooser capacityChooser;
    private SwitchableValueDistanceAndColorChooser flowChooser;

    /**
     * Konstruktor
     *
     * @param logic
     *            Verbindung zur Logik.
     * @param graph
     *            Zugrundeliegender Graph.
     * @param edge
     *            Kante deren Daten der Dialog anzeigt und ändern lässt.
     * @param coords
     *            Koordinaten, an denen der Dialog angezeigt werden soll
     * @param elements
     *            Enthält die grafischen Elemente
     */
    public EdgeDialog(Logic logic, Graph graph, Edge edge,
            PixelPoint coords, GuiElements elements) {
        super(elements.getFrameLocation(), elements.getProgramImage(),
                "Eigenschaften der Kante von Ecke " + edge.getStartVertexIndex() + " zur Ecke "
                        + edge.getTargetVertexIndex());
        addEscapeBehaviour();

        this.logic = logic;
        this.graph = graph;
        meta = graph.getMeta();
        this.originalEdge = edge;
        this.coords = coords;
        this.elements = elements;

        Graph subGraph = graph.createEdgeSubGraph(edge, SUB_GRAPH_SIZE);
        subGraphButton = new SubGraphButton(subGraph, SUB_GRAPH_SIZE);
        if (subGraph.getNumberOfEdges() != 1) {
            throw new RuntimeException("Da ging was schief bei der Erstellung des Teilgraphen...");
        }
        this.edge = subGraph.getEdge(0);

        fillDialog();
    }

    @Override
    protected void populateDialog() {
        add(createMainPart(), BorderLayout.CENTER);
        add(createButtonPart(), BorderLayout.SOUTH);

        setLocation();
    }

    private void setLocation() {
        MouseClickLocationSetter setter = new MouseClickLocationSetter(this, coords, elements);
        setter.setLocation();
    }

    private Component createMainPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(0, 2, 0, 0));

        panel.add(createLeftPanel());
        panel.add(createRightPanel());

        return panel;
    }

    private Component createLeftPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createUpperLeftPanel(), BorderLayout.CENTER);
        panel.add(createLowerLeftPanel(), BorderLayout.SOUTH);

        return panel;
    }

    private Component createUpperLeftPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(GedGuiTools.createLeftAndRightMainPartLayout());

        panel.add(createEdgeColorPanel());
        panel.add(createWidthPanel());
        panel.add(createEdgeStylePanel());
        panel.add(createLabelPanel());

        return panel;
    }

    private Component createEdgeColorPanel() {
        colorChooser = new DefaultOrOwnColorChooser(
                edge.hasDefaultColor(),
                "Default-Farbe verwenden",
                "Kantenfarbe: ",
                "Kantenfarbe für die Kante von Ecke " + edge.getStartVertexIndex() + " zur Ecke "
                        + edge.getTargetVertexIndex() + " wählen:",
                edge.hasDefaultColor() ? NOT_SHOWN_COLOR
                                        : edge.getColor().toSwingColor(),
                meta.getDefaultEdgeColor(),
                gedColor -> userSetsEdgeColor(gedColor),
                elements.getFrame());
        JPanel panel = colorChooser.createPanel();
        GuiTools.createTitle("Farbe der Kante", panel);

        return panel;
    }

    private void userSetsEdgeColor(GedColor gedColor) {
        edge.setColor(gedColor);
        if (!GedColor.isDefaultColor(gedColor)) {
            labelChooser.setDefaultButtonColor(gedColor);
            weightChooser.setDefaultButtonColor(gedColor);
            capacityChooser.setDefaultButtonColor(gedColor);
            flowChooser.setDefaultButtonColor(gedColor);
        }
        refresh();
    }

    private Component createWidthPanel() {
        widthChooser = new RadiusOrWidthChooser(edge.hasDefaultWidth(),
                "Default-Dicke verwenden",
                Edge.USE_DEFAULT_WIDTH,
                meta.getDefaultEdgeWidth(),
                "Dicke: ",
                width -> widthChanged(width),
                coords);
        widthChooser.setValue(edge.getWidth());
        widthChooser.addChangeAction(() -> widthChanged());
        JPanel panel = widthChooser.getPanel();
        GuiTools.createTitle("Dicke der Kante", panel);
        return panel;
    }

    private void widthChanged(int width) {
        edge.setWidth(width);
        refresh();
    }

    private void widthChanged() {
        if (widthChooser.isDefaultSelected()) {
            edge.setWidthToDefault();
        }
        else {
            String widthText = widthChooser.getValueText();
            if (NumberString.isDigitSequence(widthText)) {
                int width = NumberString.parseIntIgnore(widthText, -1);
                if (width > 0) {
                    edge.setWidth(width);
                }
            }
        }
        refresh();
    }

    private Component createEdgeStylePanel() {
        EdgeStyleChooser chooser = new EdgeStyleChooser(edge, createRefresher());
        return chooser.createEdgeStylePanel();
    }

    private Component createLabelPanel() {
        labelChooser = new ValueDistanceAndColorChooser(
                "Kantenbeschriftung: ",
                edge.hasDefaultLabelColor(),
                "X-Abstand der Kantenbeschriftung: ",
                "Y-Abstand der Kantenbeschriftung: ",
                "Default-Farbe für Beschriftung verwenden",
                "Farbe der Beschriftung: ",
                "Farbe der Beschriftung für die Kante von Ecke " + edge.getStartVertexIndex()
                        + " zur Ecke " + edge.getTargetVertexIndex() + " wählen:",
                edge.hasDefaultLabelColor() ? NOT_SHOWN_COLOR
                                             : edge.getLabelColor().toSwingColor(),
                edge.hasDefaultColor() ? meta.getDefaultEdgeColor()
                                        : edge.getColor(),
                gedColor -> userSetsLabelColor(gedColor),
                elements.getFrame());
        labelChooser.setStartValue(edge.getLabel());
        labelChooser.setXDistance(edge.getLabelDistanceX());
        labelChooser.setYDistance(edge.getLabelDistanceY());
        labelChooser.addChangeValueAction(() -> labelValueChanged());
        labelChooser.addChangeXDistanceAction(() -> labelXDistanceChanged());
        labelChooser.addChangeYDistanceAction(() -> labelYDistanceChanged());
        JPanel panel = labelChooser.getPanel();
        GuiTools.createTitle("Beschriftung der Kante", panel);

        return panel;
    }

    private void userSetsLabelColor(GedColor gedColor) {
        edge.setLabelColor(gedColor);
        refresh();
    }

    private void labelValueChanged() {
        edge.setLabel(labelChooser.getValue());
        refresh();
    }

    private void labelXDistanceChanged() {
        int labelX = NumberString.parseIntIgnore(labelChooser.getXDistanceFieldText(),
                Integer.MIN_VALUE);
        if (labelX > Integer.MIN_VALUE) {
            edge.setLabelDistanceX(labelX);
        }
        refresh();
    }

    private void labelYDistanceChanged() {
        int labelY = NumberString.parseIntIgnore(labelChooser.getYDistanceFieldText(),
                Integer.MIN_VALUE);
        if (labelY > Integer.MIN_VALUE) {
            edge.setLabelDistanceY(labelY);
        }
        refresh();
    }

    private Component createLowerLeftPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createSubGraphPanel(), BorderLayout.CENTER);
        panel.add(createDeleteEdgeButtonPart(), BorderLayout.SOUTH);

        return panel;
    }

    private Component createSubGraphPanel() {
        FixedCenter fixedCenter = new FixedCenter(subGraphButton);
        return fixedCenter.createPanel();
    }

    private Component createDeleteEdgeButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        GuiTools.createTitle(panel);

        panel.add(createDeleteEdgeButton(), BorderLayout.CENTER);

        return panel;
    }

    private Component createDeleteEdgeButton() {
        JButton button = new JButton("Kante löschen");
        button.addActionListener(e -> deleteEdge());
        return button;
    }

    private void deleteEdge() {
        graph.removeEdge(originalEdge);
        logic.update();
        quit();
    }

    private Component createRightPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(GedGuiTools.createLeftAndRightMainPartLayout());

        panel.add(createWeightPanel());        /* Bereich R1: Gewicht der Kante   */
        panel.add(createCapacityPanel());      /* Bereich R2: Kapazität der Kante */
        panel.add(createFlowPanel());          /* Bereich R3: Fluss der Kante     */

        return panel;
    }

    /** Erzeugt den Panel mit den Elementen zum Gewicht der Kante. */
    private Component createWeightPanel() {
        weightChooser = new SwitchableValueDistanceAndColorChooser(
                edge.hasWeight(),
                "Gewicht verwenden",
                "Gewicht der Kante: ",
                edge.hasDefaultWeightColor(),
                "X-Abstand des Gewichts: ",
                "Y-Abstand des Gewichts: ",
                "Default-Farbe für das Gewicht verwenden",
                "Farbe des Gewichts: ",
                "Farbe des Gewichts für die Kante von Ecke " + edge.getStartVertexIndex()
                        + " zur Ecke " + edge.getTargetVertexIndex() + " wählen:",
                edge.hasDefaultWeightColor() ? NOT_SHOWN_COLOR
                                              : edge.getWeightColor().toSwingColor(),
                edge.hasDefaultColor() ? meta.getDefaultEdgeColor()
                                        : edge.getColor(),
                gedColor -> userSetsWeightColor(gedColor),
                elements.getFrame());
        weightChooser.setStartValue(Double.toString(edge.getWeight()));
        weightChooser.setXDistance(edge.getWeightDistanceX());
        weightChooser.setYDistance(edge.getWeightDistanceY());
        weightChooser.addChangeValueAction(() -> weightValueChanged());
        weightChooser.addChangeXDistanceAction(() -> weightXDistanceChanged());
        weightChooser.addChangeYDistanceAction(() -> weightYDistanceChanged());
        JPanel panel = weightChooser.getPanel();
        GuiTools.createTitle("Gewicht der Kante", panel);

        return panel;
    }

    private void userSetsWeightColor(GedColor gedColor) {
        edge.setWeightColor(gedColor);
        refresh();
    }

    private void weightValueChanged() {
        String value = weightChooser.getValue();
        double weight = NumberString.parseDoubleIgnore(value, Double.MIN_VALUE);
        if (weight > Double.MIN_VALUE) {
            edge.setWeight(weight);
        }
        refresh();
    }

    private void weightXDistanceChanged() {
        int x = NumberString.parseIntIgnore(weightChooser.getXDistanceFieldText(),
                Integer.MIN_VALUE);
        if (x > Integer.MIN_VALUE) {
            edge.setWeightDistanceX(x);
        }
        refresh();
    }

    private void weightYDistanceChanged() {
        int y = NumberString.parseIntIgnore(weightChooser.getYDistanceFieldText(),
                Integer.MIN_VALUE);
        if (y > Integer.MIN_VALUE) {
            edge.setWeightDistanceY(y);
        }
        refresh();
    }

    /** Erzeugt den Panel mit den Elementen zur Kapazität der Kante. */
    private JPanel createCapacityPanel() {
        capacityChooser = new SwitchableValueDistanceAndColorChooser(
                edge.hasCapacity(),
                "Kapazität verwenden",
                "Kapazität der Kante: ",
                edge.hasDefaultCapacityColor(),
                "X-Abstand der Kapazität: ",
                "Y-Abstand der Kapazität: ",
                "Default-Farbe für die Kapazität verwenden",
                "Farbe der Kapazität: ",
                "Farbe der Kapazität für die Kante von Ecke " + edge.getStartVertexIndex()
                        + " zur Ecke " + edge.getTargetVertexIndex() + " wählen:",
                edge.hasDefaultCapacityColor() ? NOT_SHOWN_COLOR
                                                : edge.getCapacityColor().toSwingColor(),
                edge.hasDefaultColor() ? meta.getDefaultEdgeColor()
                                        : edge.getColor(),
                gedColor -> userSetsCapacityColor(gedColor),
                elements.getFrame());
        capacityChooser.setStartValue(Double.toString(edge.getCapacity()));
        capacityChooser.setXDistance(edge.getCapacityDistanceX());
        capacityChooser.setYDistance(edge.getCapacityDistanceY());
        capacityChooser.addChangeValueAction(() -> capacityValueChanged());
        capacityChooser.addChangeXDistanceAction(() -> capacityXDistanceChanged());
        capacityChooser.addChangeYDistanceAction(() -> capacityYDistanceChanged());
        JPanel panel = capacityChooser.getPanel();
        GuiTools.createTitle("Kapazität der Kante", panel);

        return panel;
    }

    private void userSetsCapacityColor(GedColor gedColor) {
        edge.setCapacityColor(gedColor);
        refresh();
    }

    private void capacityValueChanged() {
        String value = capacityChooser.getValue();
        double capacity = NumberString.parseDoubleIgnore(value, Double.MIN_VALUE);
        if (capacity > Double.MIN_VALUE) {
            edge.setCapacity(capacity);
        }
        refresh();
    }

    private void capacityXDistanceChanged() {
        int x = NumberString.parseIntIgnore(capacityChooser.getXDistanceFieldText(),
                Integer.MIN_VALUE);
        if (x > Integer.MIN_VALUE) {
            edge.setCapacityDistanceX(x);
        }
        refresh();
    }

    private void capacityYDistanceChanged() {
        int y = NumberString.parseIntIgnore(capacityChooser.getYDistanceFieldText(),
                Integer.MIN_VALUE);
        if (y > Integer.MIN_VALUE) {
            edge.setCapacityDistanceY(y);
        }
        refresh();
    }

    /** Erzeugt den Panel mit den Elementen zum Fluss der Kante. */
    private JPanel createFlowPanel() {
        flowChooser = new SwitchableValueDistanceAndColorChooser(
                edge.hasFlow(),
                "Fluss verwenden",
                "Fluss der Kante: ",
                edge.hasDefaultFlowColor(),
                "X-Abstand des Flusses: ",
                "Y-Abstand des Flusses: ",
                "Default-Farbe für den Fluss verwenden",
                "Farbe des Flusses: ",
                "Farbe des Flusses für die Kante von Ecke " + edge.getStartVertexIndex()
                        + " zur Ecke " + edge.getTargetVertexIndex() + " wählen:",
                edge.hasDefaultFlowColor() ? NOT_SHOWN_COLOR
                                            : edge.getFlowColor().toSwingColor(),
                edge.hasDefaultColor() ? meta.getDefaultEdgeColor()
                                        : edge.getColor(),
                gedColor -> userSetsFlowColor(gedColor),
                elements.getFrame());
        flowChooser.setStartValue(Double.toString(edge.getFlow()));
        flowChooser.setXDistance(edge.getFlowDistanceX());
        flowChooser.setYDistance(edge.getFlowDistanceY());
        flowChooser.addChangeValueAction(() -> flowValueChanged());
        flowChooser.addChangeXDistanceAction(() -> flowXDistanceChanged());
        flowChooser.addChangeYDistanceAction(() -> flowYDistanceChanged());
        JPanel panel = flowChooser.getPanel();
        GuiTools.createTitle("Fluss auf der Kante", panel);

        return panel;
    }
    private void flowValueChanged() {
        String value = flowChooser.getValue();
        double flow = NumberString.parseDoubleIgnore(value, Double.MIN_VALUE);
        if (flow > Double.MIN_VALUE) {
            edge.setFlow(flow);
        }
        refresh();
    }

    private void flowXDistanceChanged() {
        int x = NumberString.parseIntIgnore(flowChooser.getXDistanceFieldText(),
                Integer.MIN_VALUE);
        if (x > Integer.MIN_VALUE) {
            edge.setFlowDistanceX(x);
        }
        refresh();
    }

    private void flowYDistanceChanged() {
        int y = NumberString.parseIntIgnore(flowChooser.getYDistanceFieldText(),
                Integer.MIN_VALUE);
        if (y > Integer.MIN_VALUE) {
            edge.setCapacityDistanceY(y);
        }
        refresh();
    }

    private void userSetsFlowColor(GedColor gedColor) {
        edge.setFlowColor(gedColor);
        refresh();
    }

    private Component createButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createQuitButton(), BorderLayout.WEST);
        panel.add(createOkButton(), BorderLayout.EAST);

        return panel;
    }

    private Component createQuitButton() {
        JButton button = new JButton("Abbrechen");
        button.addActionListener(e -> quit());
        return button;
    }

    private Component createOkButton() {
        JButton button = new JButton("OK");
        button.addActionListener(e -> pressedOkButton());
        return button;
    }

    /** Reagiert auf den Druck auf den OK-Button. */
    private void pressedOkButton() {
        copyValues();
        quit();
    }

    private void copyValues() {
        int startVertexIndex = originalEdge.getStartVertexIndex();
        int targetVertexIndex = originalEdge.getTargetVertexIndex();

        originalEdge.copyValuesFrom(edge);

        originalEdge.setStartVertexIndex(startVertexIndex);
        originalEdge.setTargetVertexIndex(targetVertexIndex);
    }

    private void quit() {
        setVisible(false);
        dispose();
    }

}
