package de.duehl.math.graph.ged.ui.graphpanel;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Stroke;

import de.duehl.basics.text.NumberString;
import de.duehl.math.geometry.Geometry;
import de.duehl.math.geometry.Line;
import de.duehl.math.geometry.Point;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.EdgeStyle;
import de.duehl.math.graph.ged.graph.edge.Edges;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.pixel.PixelPoint;

/**
 * Diese Klasse steht zeichnet Graphen in das Panel ein.
 *
 * @version 1.01     2019-06-14
 * @author Christian Dühl
 */

public class GraphPainter {

    /** Grafikobjekt in das gezeichnet werden soll */
    private Graphics graphics;

    /** Darzustellender Graph */
    private Graph graph;

    /** Metadaten des darzustellenden Graphens (zur Bequemlichkeit). */
    private MetaData meta;

    private boolean fillArrows;

    private boolean drawArrows;

    /** Konstruktor */
    public GraphPainter() {
        graphics = null;
        graph = null;
        fillArrows = true;
        drawArrows = true;
    }

// Braucht man vielleicht noch, wenn umgeschaltet wird, ob die Pfeile
// dargestellt werden sollen:
//    public boolean isFillArrows() {
//        return fillArrows;
//    }
//
//    public void setFillArrows(boolean fillArrows) {
//        this.fillArrows = fillArrows;
//    }
//
//    public boolean isDrawArrows() {
//        return drawArrows;
//    }
//
//    public void setDrawArrows(boolean drawArrows) {
//        this.drawArrows = drawArrows;
//    }

    /**
     * Zeichnet den Graphen.
     * @param graphics
     *            Grafikobjekt in das gezeichnet werden soll
     * @param graph
     */
    public void paintGraph(Graphics graphics, Graph graph) {
        this.graphics = graphics;
        this.graph = new Graph(graph); // um eine Kopie darzustellen
        meta = this.graph.getMeta();

        /* Ecken einzeichnen: */
        paintVertices();

        /* Kanten einzeichnen: */
        paintEdges();
    }

    /** Die Ecken auf das Panel zeichnen. */
    private void paintVertices() {
        for (Vertex vertex : graph.getVertices()) {
            paintVertex(vertex);
        }
    }

    private void paintVertex(Vertex vertex) {
        PixelPoint coordinates = vertex.getCoordinates();
        int x = coordinates.getX();
        int y = coordinates.getY();
        int radius = radiusOrDefaultRadius(vertex.getRadius());

        GedColor vertexColor = colorOrDefaultColor(vertex.getColor(), meta.getDefaultVertexColor());
        java.awt.Color color = vertexColor.toSwingColor();
        graphics.setColor(color);
        graphics.fillOval(x-radius, y-radius, 2*radius, 2*radius);

        /* Eckenbezeichnung darstellen: */
        paintVertexLabel(vertex, vertexColor);

        /* Eckengewicht darstellen: */
        paintVertexWeight(vertex, vertexColor);
    }

    private int radiusOrDefaultRadius(int radius) {
        if (radius == Vertex.USE_DEFAULT_RADIUS) {
            return meta.getDefaultRadius();
        }
        else {
            return radius;
        }
    }

    private GedColor colorOrDefaultColor(GedColor color, GedColor defaultColor) {
        if (GedColor.isDefaultColor(color)) {
            return defaultColor;
        }
        else {
            return color;
        }
    }

    /**
     * Stellt die Eckenbezeichnung dar.
     *
     * @param vertex
     *            Ecke, deren Bezeichnung dargstellt werden soll.
     * @param vertexColor
     *            Für die Ecke zu verwendende Farbe.
     */
    private void paintVertexLabel(Vertex vertex, GedColor vertexColor) {
        if (vertex.hasLabel()) {
            reallyPaintVertexLabel(vertex, vertexColor);
        }
    }

    private void reallyPaintVertexLabel(Vertex vertex, GedColor vertexColor) {
        String label = vertex.getLabel();
        GedColor labelColor = colorOrDefaultColor(vertex.getLabelColor(), vertexColor);
        PixelPoint coordinates = vertex.getCoordinates();
        PixelPoint distance = vertex.getLabelDistance();
        int labelX = coordinates.getX() + distance.getX();
        int labelY = coordinates.getY() + distance.getY();
        graphics.setColor(labelColor.toSwingColor());
        graphics.drawString(label, labelX, labelY);
    }

    /**
     * Stellt das Gewicht der Ecke dar.
     *
     * @param vertex
     *            Ecke, deren Bezeichnung dargstellt werden soll.
     * @param vertexColor
     *            Für die Ecke zu verwendende Farbe.
     */
    private void paintVertexWeight(Vertex vertex, GedColor vertexColor) {
        if (vertex.hasWeight()) {
            reallyPaintVertexWeight(vertex, vertexColor);
        }
    }

    private void reallyPaintVertexWeight(Vertex vertex, GedColor vertexColor) {
        double weight = vertex.getWeight();
        String weightText = NumberString.removeZeroEnding(Double.toString(weight));

        GedColor weightColor = colorOrDefaultColor(vertex.getWeightColor(), vertexColor);
        PixelPoint coordinates = vertex.getCoordinates();
        PixelPoint distance = vertex.getWeightDistance();
        int labelX = coordinates.getX() + distance.getX();
        int labelY = coordinates.getY() + distance.getY();
        graphics.setColor(weightColor.toSwingColor());
        graphics.drawString(weightText, labelX, labelY);
    }

    /** Die Kanten auf das Panel zeichnen. */
    private void paintEdges() {
        for (Vertex vertex : graph.getVertices()) {
            paintEdgesBeginningAtVertex(vertex);
        }
    }

    private void paintEdgesBeginningAtVertex(Vertex vertex) {
        /* Startpunkt ist der Mittelpunkt der Ausgangsecke: */
        PixelPoint start = vertex.getCoordinates();

        /* Radius der Startecke ermitteln: */
        int startRadius = radiusOrDefaultRadius(vertex.getRadius());

        Edges forwardNeighbours = graph.getEdgesStartingFromVertex(vertex);
        for (Edge neighbourEntry : forwardNeighbours) {
            int neighbourIndex = neighbourEntry.getTargetVertexIndex();
            Vertex neighbour = graph.getVertex(neighbourIndex);

            /* Endpunkt ist der Mittelpunkt der Zielecke: */
            PixelPoint end = neighbour.getCoordinates();

            /* Radius des Endpunktes ermitteln: */
            int endRadius = radiusOrDefaultRadius(neighbour.getRadius());

            /* Verbindungslinie zwischen Start- und Endpunkt bilden: */
            Line line = new Line(start.toMathPoint(), end.toMathPoint());

            /* Falls die Linie lang genug ist, Kante darstellen: */
            if (line.getDistance() > startRadius + endRadius) {
                paintEdge(neighbourEntry, startRadius, endRadius, line);
            }
        }
    }

    /**
     * Kante darstellen.
     *
     * @param entry
     *            Adjazenz-Element der Kante, um die es geht.
     * @param startRadius
     *            Radius der Startecke
     * @param endRadius
     *            Radius der Endecke
     * @param line
     *            Verbindungslinie zwischen Start- und Endpunkt
     */
    private void paintEdge(Edge entry, int startRadius, int endRadius, Line line) {
        /* Farbe der Kante bestimmen: */
        GedColor color = colorOrDefaultColor(entry.getColor(), meta.getDefaultEdgeColor());

        /* Dicke der Kante bestimmen: */
        int width = widthOrDefaultWidth(entry.getWidth());

        /*
         * Bestimmung des Start- und Endpunktes der Linie an den Rändern der
         * Kreisdarstellungen der Ecken:
         */
        Geometry geometry = new Geometry();
        Point realStart = geometry.getPointOnLineWithDistanceToStartPoint(line, startRadius);
        Point realEnd = geometry.getPointOnLineWithDistanceToEndPoint(line, endRadius);

        /*
         * Bei dicken Kanten mit Spitze fällt auf, dass die Kante über die Spitze heraus ragt. Also
         * passend kürzer zeichnen. Dafür wird nun hier adjustEnd und realEndForArrow gebildet und
         * verwendet.
         *
         * TODO: Zu beachten ist noch, dass eine Kante in Gegenrichtung vorhanden sein könnte. In
         * diesem Fall muss auf der Anfang entsprechend verkürzt werden.
         */
        int adjustEnd = endRadius;
        if (drawArrows && graph.isDirected()) {
            adjustEnd += getArrowLength(width) + 5; // TODO ...
        }
        Point realEndForArrow = geometry.getPointOnLineWithDistanceToEndPoint(line, adjustEnd);

        /*
         * Die Linie realLine verbindet den Schnittpunkt der Kreisdarstellung
         * der Startecke und der Linie zwischen den Mittelpunkten der Start- und
         * der Endecke mit dem Schnittpunkt der Kreisdarstellung der Endecke mit
         * eben der Verbindungslinie.
         */
        Line realLine = new Line(realStart, realEnd);
        Line realLineForArrow = new Line(realStart, realEndForArrow);

        /* Kante zeichnen: */
        EdgeStyle style = entry.getStyle();
        drawLine(realLineForArrow, color.toSwingColor(), style, width);

        /* Pfeilspitzen zeichnen: */
        if (drawArrows && graph.isDirected()) {
            paintArrow(realLine, color.toSwingColor(), style, width);
        }

        /* Kantenbezeichnung, Gewicht, Kapazität und Fluss darstellen: */
        Point centerPoint = geometry.calculateCenter(realStart, realEnd);
        PixelPoint center = new PixelPoint(centerPoint);
        paintEdgeLabel(entry, color, center);
        paintEdgeWeight(entry, color, center);
        paintEdgeCapacity(entry, color, center);
        paintEdgeFlow(entry, color, center);
    }

    private int widthOrDefaultWidth(int width) {
        if (width == Edge.USE_DEFAULT_WIDTH) {
            return meta.getDefaultEdgeWidth();
        }
        else {
            return width;
        }
    }

    /**
     * Stellt die Bezeichnung der Kante dar.
     *
     * @param entry
     *            Adjazenz-Element der Kante, um die es geht.
     * @param color
     *            Farbe der Kante.
     * @param center
     *            Mittelpunkt der Kante unter berücksichtigung der Radien der
     *            Eckendarstellungen.
     */
    private void paintEdgeLabel(Edge entry, GedColor color, PixelPoint center) {
        if (entry.hasLabel()) {
            reallyPaintEdgeLabel(entry, color, center);
        }
    }

    private void reallyPaintEdgeLabel(Edge entry, GedColor color, PixelPoint center) {
        String label = entry.getLabel();
        GedColor labelColor = colorOrDefaultColor(entry.getLabelColor(), color);
        PixelPoint distance = entry.getLabelDistance();
        int labelX = center.getX() + distance.getX();
        int labelY = center.getY() + distance.getY();
        graphics.setColor(labelColor.toSwingColor());
        graphics.drawString(label, labelX, labelY);
    }

    /**
     * Stellt das Gewicht der Kante dar.
     *
     * @param entry
     *            Adjazenz-Element der Kante, um die es geht.
     * @param color
     *            Farbe der Kante.
     * @param center
     *            Mittelpunkt der Kante unter berücksichtigung der Radien der
     *            Eckendarstellungen.
     */
    private void paintEdgeWeight(Edge entry, GedColor color, PixelPoint center) {
        if (entry.hasWeight()) {
            reallyPaintEdgeWeight(entry, color, center);
        }
    }

    private void reallyPaintEdgeWeight(Edge entry, GedColor color, PixelPoint center) {
        double weight = entry.getWeight();
        String weightText = NumberString.removeZeroEnding(Double.toString(weight));
        GedColor weightColor = colorOrDefaultColor(entry.getWeightColor(), color);
        PixelPoint distance = entry.getWeightDistance();
        int labelX = center.getX() + distance.getX();
        int labelY = center.getY() + distance.getY();
        graphics.setColor(weightColor.toSwingColor());
        graphics.drawString(weightText, labelX, labelY);
    }

    /**
     * Stellt die Kapazität der Kante dar.
     *
     * @param entry
     *            Adjazenz-Element der Kante, um die es geht.
     * @param color
     *            Farbe der Kante.
     * @param center
     *            Mittelpunkt der Kante unter berücksichtigung der Radien der
     *            Eckendarstellungen.
     */
    private void paintEdgeCapacity(Edge entry, GedColor color, PixelPoint center) {
        if (entry.hasCapacity()) {
            reallyPaintEdgeCapacity(entry, color, center);
        }
    }

    private void reallyPaintEdgeCapacity(Edge entry, GedColor color, PixelPoint center) {
        double capacity = entry.getCapacity();
        String capacityText = NumberString.removeZeroEnding(Double.toString(capacity));
        GedColor capacityColor = colorOrDefaultColor(entry.getCapacityColor(), color);
        PixelPoint distance = entry.getCapacityDistance();
        int labelX = center.getX() + distance.getX();
        int labelY = center.getY() + distance.getY();
        graphics.setColor(capacityColor.toSwingColor());
        graphics.drawString(capacityText, labelX, labelY);
    }

    /**
     * Stellt den Fluss der Kante dar.
     *
     * @param entry
     *            Adjazenz-Element der Kante, um die es geht.
     * @param color
     *            Farbe der Kante.
     * @param center
     *            Mittelpunkt der Kante unter berücksichtigung der Radien der
     *            Eckendarstellungen.
     */
    private void paintEdgeFlow(Edge entry, GedColor color, PixelPoint center) {
        if (entry.hasFlow()) {
            reallyPaintEdgeFlow(entry, color, center);
        }
    }

    private void reallyPaintEdgeFlow(Edge entry, GedColor color, PixelPoint center) {
        double flow = entry.getFlow();
        String flowText = NumberString.removeZeroEnding(Double.toString(flow));
        GedColor flowColor = colorOrDefaultColor(entry.getFlowColor(), color);
        PixelPoint distance = entry.getFlowDistance();
        int labelX = center.getX() + distance.getX();
        int labelY = center.getY() + distance.getY();
        graphics.setColor(flowColor.toSwingColor());
        graphics.drawString(flowText, labelX, labelY);
    }

    /**
     * Zeichnet eine Linie auf das Panel.
     *
     * @param line
     *            Einzuzeichnende Linie
     * @param color
     *            Farbe der Linie
     * @param style
     *            Stil der Linie.
     * @param width
     *            Dicke der Linie.
     */
    public void drawLine(Line line, GedColor color, EdgeStyle style, int width) {
        drawLine(line, color.toSwingColor(), style, width);
    }

    /**
     * Zeichnet eine Linie auf das Panel.
     *
     * @param line
     *            Einzuzeichnende Linie
     * @param color
     *            Farbe der Linie
     * @param style
     *            Stil der Linie.
     * @param width
     *            Dicke der Linie.
     */
    private void drawLine(Line line, java.awt.Color color, EdgeStyle style, int width) {
        Stroke stroke = generateStrokeFromStyle(style, width);
        Graphics2D g2d = (Graphics2D) graphics;
        g2d.setStroke(stroke);
        g2d.setColor(color);
        Point realStart = line.getStart();
        Point realEnd = line.getEnd();
        g2d.drawLine(
                realStart.getXAsInteger(),
                realStart.getYAsInteger(),
                realEnd.getXAsInteger(),
                realEnd.getYAsInteger()
        );
    }

    /**
     * Erzeugt aus der Dicke der Linie und dem Stil ein BasicStroke.
     *
     * @param style
     *            Stil der Linie
     * @param width
     *            Dicke der Linie
     * @return BasicStroke für die Darstellung
     */
    private Stroke generateStrokeFromStyle(EdgeStyle style, int width) {
        Stroke stroke;
        switch (style) {
            default:
            case NORMAL:
                stroke = new BasicStroke((float) width);
                break;
            case DASHED:
                stroke = new BasicStroke(
                        (float) width,              // Width
                        BasicStroke.CAP_BUTT,       // End cap
                        BasicStroke.JOIN_MITER,     // Join style
                        10.0f,                      // Miter limit
                        new float[] {8.0f, 8.0f},   // Dash pattern
                        0.0f);                      // Dash phase
                    break;
            case DOTTED:
                stroke = new BasicStroke(
                        (float) width,              // Width
                        BasicStroke.CAP_BUTT,       // End cap
                        BasicStroke.JOIN_MITER,     // Join style
                        10.0f,                      // Miter limit
                        new float[] {4.0f, 4.0f},   // Dash pattern
                        0.0f);                      // Dash phase
                    break;
        }
        //new float[] { 21.0f, 9.0f, 3.0f, 9.0f }, // Dot-Dash pattern
        return stroke;
    }

    /**
     * Zeichnet eine Spitze an eine Linie.
     *
     * @param line
     *            Linie, an deren Ende eine Spitze gezeichnet werden soll.
     * @param color
     *            Farbe der Spitze
     * @param style
     *            Stil der Linie.
     * @param width
     *            Dicke der Linie.
     */
    public void paintArrow(Line line, GedColor color, EdgeStyle style, int width) {
        java.awt.Color swingColor = color.toSwingColor();
        paintArrow(line, swingColor, style, width);
    }

    /**
     * Zeichnet eine Spitze an eine Linie.
     *
     * @param line
     *            Linie, an deren Ende eine Spitze gezeichnet werden soll.
     * @param color
     *            Farbe der Spitze
     * @param style
     *            Stil der Linie.
     * @param width
     *            Dicke der Linie.
     */
    private void paintArrow(Line line, java.awt.Color color, EdgeStyle style, int width) {
        PixelPoint start = new PixelPoint(line.getStart());
        PixelPoint end = new PixelPoint(line.getEnd());

        int headWidth  = getArrowWidth(width);
        int headLength = getArrowLength(width);
        headWidth/=2;

        /* Pfeilspitze: */
        double ax = end.getX() - start.getX();
        double ay = end.getY() - start.getY();
        double aLen = Math.sqrt(ax*ax + ay*ay);

        ax /= aLen;
        ay /= aLen;

        Point p1 = end.toMathPoint();
        Point p2 = new Point(
                end.getX() - headLength * ax + headWidth * ay,
                end.getY() - headLength * ay - headWidth * ax
        );
        Point p3 = new Point(
                end.getX() - headLength * ax - headWidth * ay,
                end.getY() - headLength * ay + headWidth * ax
        );

        if (fillArrows) {
            Polygon p = new Polygon();
            p.addPoint(p1.getXAsInteger(), p1.getYAsInteger());
            p.addPoint(p2.getXAsInteger(), p2.getYAsInteger());
            p.addPoint(p3.getXAsInteger(), p3.getYAsInteger());
            graphics.fillPolygon(p);
        }
        else {
            // Hier dennoch normal!
            drawLine(new Line(p1, p2), color, EdgeStyle.NORMAL, width);
            drawLine(new Line(p1, p3), color, EdgeStyle.NORMAL, width);
            drawLine(new Line(p2, p3), color, EdgeStyle.NORMAL, width);
        }
    }

    /**
     * Berechnet die Länge der Pfeilspitze.
     *
     * @param width
     *            Dicke der zugehörigen Kante.
     * @return Länge
     */
    private int getArrowLength(int width) {
        return 10 + width;
    }

    /**
     * Berechnet die Breite der Pfeilspitze.
     *
     * @param width
     *            Dicke der zugehörigen Kante.
     * @return Länge
     */
    private int getArrowWidth(int width) {
        return 15 + width;
    }

}
