package de.duehl.math.graph.ged.graph;

import static org.junit.jupiter.api.Assertions.*;

import java.util.Random;

import org.junit.Test;

import de.duehl.basics.text.Text;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

public class GraphTest {

    @Test
    public void createEmptyGraph() {
        Graph graph = new Graph();
        assertNotNull(graph);
    }

    @Test
    public void noVerticesInEmptyGraph() {
        Graph graph = new Graph();
        int numberOfVertices = graph.getNumberOfVertices();
        assertEquals(0, numberOfVertices);
    }

    @Test
    public void noEdgesInEmptyGraph() {
        Graph graph = new Graph();
        int numberOfEdges = graph.getNumberOfEdges();
        assertEquals(0, numberOfEdges);
    }

    @Test
    public void addOneVertexAndNIs1() {
        Graph graph = new Graph();
        Vertex vertex = new Vertex();
        vertex.setLabel("Ecke eins");
        graph.addVertex(vertex);
        int numberOfVertices = graph.getNumberOfVertices();
        assertEquals(1, numberOfVertices);
    }

    @Test
    public void addOneVertexAndItsIndexDoChange() {
        Graph graph = new Graph();
        Vertex vertex = new Vertex();
        vertex.setLabel("Ecke eins");
        vertex.setIndex(77); // dies sollten Anwender nicht tun, und können sie
                             // wegen der Sichtbarkeit auch nicht!
        graph.addVertex(vertex);
        int index = vertex.getIndex();
        assertEquals(0, index);
    }

    @Test
    public void addTwoVerticesAndNIs2() {
        Graph graph = new Graph();
        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex1);
        graph.addVertex(vertex2);
        int numberOfVertices = graph.getNumberOfVertices();
        assertEquals(2, numberOfVertices);
    }

    @Test
    public void failsToAddOneEdgeWithoutVertices() {
        Graph graph = new Graph();
        Edge edge = new Edge(12, 24);
        assertThrows(IllegalArgumentException.class, () -> graph.addEdge(edge));
    }

    @Test
    public void failsToAddOneEdgeWithoutStartVertex() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Edge edge = new Edge(1, 0);
        assertThrows(IllegalArgumentException.class, () -> graph.addEdge(edge));
    }

    @Test
    public void addOneEdgeWithoutEndVertex() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Edge edge = new Edge(0, 1);
        assertThrows(IllegalArgumentException.class, () -> graph.addEdge(edge));
    }

    @Test
    public void addTwoVerticesAndOneEdgeAndEdgeNumberIs1() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        Edge edge = new Edge(0, 1);
        graph.addEdge(edge);

        int numberOfVertices = graph.getNumberOfVertices();
        assertEquals(2, numberOfVertices);
        int numberOfEdges = graph.getNumberOfEdges();
        assertEquals(1, numberOfEdges);
    }

    @Test
    public void addTwoVerticesAndOneEdgeAndCheckIfEdgeExists() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        graph.addEdge(0, 1);

        assertTrue(graph.existsEdge(vertex1, vertex2));
        assertFalse(graph.existsEdge(vertex2, vertex1));
    }

    @Test
    public void addTwoVerticesAndOneEdgeAndCheckIfEdgeExistsViaIndex() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        graph.addEdge(0, 1);

        assertTrue(graph.existsEdge(0, 1));
        assertFalse(graph.existsEdge(1, 0));
    }

    @Test
    public void addTwoVerticesAndOneEdgeAndDoToString() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        Edge edge = new Edge(0, 1);
        graph.addEdge(edge);

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 2" + n
                + "    Anzahl Kanten: 1" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke zwei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void addTwoVerticesAndOneEdgeViaVertexIndicesAndDoToString() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        graph.addEdge(0, 1);

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 2" + n
                + "    Anzahl Kanten: 1" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke zwei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void addTwoVerticesAndOneEdgeViaVerticesAndDoToString() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        graph.addEdge(vertex1, vertex2);

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 2" + n
                + "    Anzahl Kanten: 1" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke zwei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void addOneVertexAndRemoveOneAndVertexNumberIs0() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        graph.removeVertex(0);

        int numberOfVertices = graph.getNumberOfVertices();
        assertEquals(0, numberOfVertices);
    }

    @Test
    public void addOneVertexAndRemoveOneAndVertexIndexIsMinusOne() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);
        assertEquals(0, vertex1.getIndex());

        graph.removeVertex(0);
        assertEquals(-1, vertex1.getIndex());
    }

    @Test
    public void addVertexRemoveAndAddAgain() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);
        assertEquals(0, vertex1.getIndex());

        graph.removeVertex(0);
        assertEquals(-1, vertex1.getIndex());

        graph.addVertex(vertex1);
        assertEquals(0, vertex1.getIndex());
    }

    @Test
    public void addIdenticalVertexTwoTimesAndDoNotGetError() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke eins");
        graph.addVertex(vertex2);
    }

    @Test
    public void addEqualVertexTwoTimes() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke eins");
        graph.addVertex(vertex2);

        assertEquals(0, vertex1.getIndex());
        assertEquals(1, vertex2.getIndex());
    }

    @Test
    public void addTwoVerticesAndRemoveOneAndVertexNumberIs1() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke eins");
        graph.addVertex(vertex2);

        graph.removeVertex(0);

        int numberOfVertices = graph.getNumberOfVertices();
        assertEquals(1, numberOfVertices);
        assertEquals(-1, vertex1.getIndex());
        assertEquals(0, vertex2.getIndex());
    }

    @Test
    public void triangleToString() {
        Graph graph = createTriangle();

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 3" + n
                + "    Anzahl Kanten: 3" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke zwei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=2," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke drei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=2," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=2," + n
                + "            targetVertexIndex=0," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    private Graph createTriangle() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        Vertex vertex3 = new Vertex();
        vertex3.setLabel("Ecke drei");
        graph.addVertex(vertex3);

        graph.addEdge(0, 1);
        graph.addEdge(1, 2);
        graph.addEdge(2, 0);
        return graph;
    }

    @Test
    public void complexVertexRemove1() {
        Graph graph = createTriangle();

        assertEquals(3, graph.getNumberOfVertices());
        assertEquals(3, graph.getNumberOfEdges());

        graph.removeVertex(1);
        assertEquals(2, graph.getNumberOfVertices());
        assertEquals(1, graph.getNumberOfEdges());

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 2" + n
                + "    Anzahl Kanten: 1" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke drei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=0," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void buildBidirectionalTriangleAndDoToString() {
        Graph graph = createBidirectionalTriangle();

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 3" + n
                + "    Anzahl Kanten: 6" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke zwei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=2," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke drei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=2," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=2," + n
                + "            targetVertexIndex=0," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=2," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=2," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=0," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    private Graph createBidirectionalTriangle() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        Vertex vertex3 = new Vertex();
        vertex3.setLabel("Ecke drei");
        graph.addVertex(vertex3);

        graph.addEdge(0, 1);
        graph.addEdge(1, 2);
        graph.addEdge(2, 0);

        graph.addEdge(0, 2);
        graph.addEdge(2, 1);
        graph.addEdge(1, 0);

        return graph;
    }

    @Test
    public void complexVertexRemove2() {
        Graph graph = createBidirectionalTriangle();

        assertEquals(3, graph.getNumberOfVertices());
        assertEquals(6, graph.getNumberOfEdges());

        graph.removeVertex(1);
        assertEquals(2, graph.getNumberOfVertices());
        assertEquals(2, graph.getNumberOfEdges());

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 2" + n
                + "    Anzahl Kanten: 2" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke drei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=0," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void complexVertexRemove2b() {
        Graph graph = createBidirectionalTriangle();

        assertEquals(3, graph.getNumberOfVertices());
        assertEquals(6, graph.getNumberOfEdges());

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");

        Vertex vertex3 = new Vertex();
        vertex3.setLabel("Ecke drei");

        Vertex actualVertex1 = graph.getVertex(0);
        Vertex actualVertex2 = graph.getVertex(1);
        Vertex actualVertex3 = graph.getVertex(2);

        assertTrue(vertex1.equalsIgnoreIndex(actualVertex1));
        assertTrue(vertex2.equalsIgnoreIndex(actualVertex2));
        assertTrue(vertex3.equalsIgnoreIndex(actualVertex3));

        graph.removeVertex(actualVertex2);
        assertEquals(2, graph.getNumberOfVertices());
        assertEquals(2, graph.getNumberOfEdges());
        assertEquals(0, actualVertex1.getIndex());
        assertEquals(-1, actualVertex2.getIndex());
        assertEquals(1, actualVertex3.getIndex());

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 2" + n
                + "    Anzahl Kanten: 2" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke drei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=0," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    @Test
    public void complexGraph01() {
        Graph graph = createBidirectionalComplexGraph();

        assertEquals(5, graph.getNumberOfVertices());
        assertEquals(8, graph.getNumberOfEdges()); /* 2 * 4 */

        String n = Text.LINE_BREAK;
        String excpected = ""
                + "Graph:" + n
                + "    Anzahl Ecken : 5" + n
                + "    Anzahl Kanten: 8" + n
                + n
                + "    Ecken:" + n
                + "        Vertex [" + n
                + "            index=0," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke eins," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=1," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke zwei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=2," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke drei," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=3," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke vier," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Vertex [" + n
                + "            index=4," + n
                + "            coordinates=PixelPoint [x=0, y=0]," + n
                + "            radius=-16," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            label=Ecke fünf," + n
                + "            labelDistance=PixelPoint [x=0, y=0]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=0, y=0]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    Kanten:" + n
                + "        Edge [" + n
                + "            startVertexIndex=0," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=0," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=1," + n
                + "            targetVertexIndex=4," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=4," + n
                + "            targetVertexIndex=1," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=3," + n
                + "            targetVertexIndex=4," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=4," + n
                + "            targetVertexIndex=3," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=2," + n
                + "            targetVertexIndex=3," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + "        Edge [" + n
                + "            startVertexIndex=3," + n
                + "            targetVertexIndex=2," + n
                + "            color=Color [red=-1, blue=-1, green=-1]," + n
                + "            style=NORMAL," + n
                + "            width=-17," + n
                + "            label=," + n
                + "            labelDistance=PixelPoint [x=-20, y=-20]," + n
                + "            labelColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            weight=-Infinity," + n
                + "            weightDistance=PixelPoint [x=20, y=20]," + n
                + "            weightColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            capacity=-Infinity," + n
                + "            capacityDistance=PixelPoint [x=-20, y=20]," + n
                + "            capacityColor=Color [red=-1, blue=-1, green=-1]," + n
                + "            flow=-Infinity," + n
                + "            flowDistance=PixelPoint [x=20, y=-20]," + n
                + "            flowColor=Color [red=-1, blue=-1, green=-1]" + n
                + "        ]" + n
                + n
                + "    MetaData:" + n
                + "        Art des Graphen    : gerichtet" + n
                + "        Default Eckenfarbe : Color [red=255, blue=0, green=0]" + n
                + "        Default Kantenfarbe: Color [red=0, blue=0, green=0]" + n
                + "        Default Radius     : 4" + n
                + "        Hintergrundfarbe   : Color [red=255, blue=255, green=255]" + n
                + "        Hintergrundbild    : " + n
                + "        Gitter anzeigen    : false" + n
                + "        Gitterabstand      : 20" + n
                + "        Einrasten am Gitter: false" + n
                + "        Farbe des Gitters  : Color [red=211, blue=211, green=211]" + n
                + "        Default Kantendicke: 1" + n;

        String actual = graph.toString();
        assertEquals(excpected, actual);
    }

    /**
     * Erstellt den folgenden ungerichteten (bzw. bidirektionalen) Graphen:
     *
     *     3---4
     *           \
     *            5
     *           /
     *     1---2
     */
    private Graph createBidirectionalComplexGraph() {
        Graph graph = new Graph();

        Vertex vertex1 = new Vertex();
        vertex1.setLabel("Ecke eins");
        graph.addVertex(vertex1);

        Vertex vertex2 = new Vertex();
        vertex2.setLabel("Ecke zwei");
        graph.addVertex(vertex2);

        Vertex vertex3 = new Vertex();
        vertex3.setLabel("Ecke drei");
        graph.addVertex(vertex3);

        Vertex vertex4 = new Vertex();
        vertex4.setLabel("Ecke vier");
        graph.addVertex(vertex4);

        Vertex vertex5 = new Vertex();
        vertex5.setLabel("Ecke fünf");
        graph.addVertex(vertex5);

        /* 1--2 */
        graph.addEdge(vertex1, vertex2);
        graph.addEdge(vertex2, vertex1);

        /* 2--5 */
        graph.addEdge(vertex2, vertex5);
        graph.addEdge(vertex5, vertex2);

        /* 4--5 */
        graph.addEdge(vertex4, vertex5);
        graph.addEdge(vertex5, vertex4);

        /* 3--4 */
        graph.addEdge(vertex3, vertex4);
        graph.addEdge(vertex4, vertex3);

        return graph;
    }

    @Test
    public void randomGraph() {
        Graph graph = new Graph();
        Random random = new Random();
        int numberOfVertices = 3 + random.nextInt(8); // 3 - 10 Ecken
        for (int vertexIndex = 0; vertexIndex < numberOfVertices; ++vertexIndex) {
            Vertex vertex = new Vertex();
            vertex.setLabel("Ecke " + (vertexIndex + 1));
            graph.addVertex(vertex);
        }
        int numberOfEdges = 2 +  + random.nextInt(3); // 2 - 4 ungerichtete Kanten
        for (int edgeIndex = 0; edgeIndex < numberOfEdges; ++edgeIndex) {
            boolean added = false;
            while (!added) {
                int v1 = random.nextInt(numberOfVertices);
                int v2 = random.nextInt(numberOfVertices);
                /* keine Schleifen: */
                if (v1 == v2) {
                    continue;
                }
                /* keine vorhandenen Kanten: */
                if (graph.existsEdge(v1, v2)) {
                    continue;
                }
                graph.addEdge(v1, v2);
                graph.addEdge(v2, v1);
                added = true;
            }
        }

        assertEquals(numberOfVertices, graph.getNumberOfVertices());
        assertEquals(2 * numberOfEdges, graph.getNumberOfEdges());
    }

}
