package de.duehl.math.graph.ged.graph.io;

import static de.duehl.math.graph.ged.graph.io.Ged4LoadAndSaveConstants.*;

import java.util.Arrays;
import java.util.List;

import de.duehl.basics.exceptions.WrongSourceFormatException;
import de.duehl.basics.text.NumberString;
import de.duehl.basics.text.Text;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.edge.AdjacenceEntry;
import de.duehl.math.graph.ged.graph.edge.EdgeStyle;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

/**
 * Diese Klasse lädt einen Graphen aus dem .ged4-Format.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class Ged4FormatLoader {

    /** Der Graph in den hier geladen wird. */
    final private Graph graph;

    /** Enthält weitere Informationen zum Graphen. */
    final private MetaData meta; // zur Bequemlichkeit.

    /**
     * Konstruktor.
     * @param graph Der Graph in den hier geladen wird.
     */
    public Ged4FormatLoader(Graph graph) {
        this.graph = graph;
        meta = graph.getMeta();
    }

    /**
     * Lädt den Graphen aus dem .ged4-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     * @throws WrongSourceFormatException
     *             Falls das Format nicht verstanden wird.
     */
    public void loadFromSaveString(String text) {
        graph.clearGraph();

        List<String> lines = Text.splitByLineBreaks(text);

        for (String line : lines) {
            workOnLine(line);
        }
    }

    private void workOnLine(String line) {
        List<String> parts = Text.splitBySemicolon(line);
        if (parts.size() >= 1) {
            String firtsPart = parts.get(0);
            parts.remove(0);
            if (META_LINE_START.equals(firtsPart)) {
                workOnMetaLine(line, parts);
            }
            else if (VERTEX_LINE_START.equals(firtsPart)) {
                workOnVertexLine(line, parts);
            }
            else if (EDGE_LINE_START.equals(firtsPart)) {
                workOnEdgeLine(line, parts);
            }
            else if (!line.isEmpty()) {
                throw new WrongSourceFormatException("Unbekannter " + "Zeilenanfang '" + firtsPart
                        + "' in Zeile\n\t" + line + "\tim ged4-Format!");
            }
        }
    }

    private void workOnMetaLine(String line, List<String> parts) {
        for (String part : parts) {
            List<String> equationParts = Arrays.asList(part.split("=", -1));
            if (equationParts.size() != 2) {
                throw new WrongSourceFormatException("Nicht erlaubte Anzahl in Gleichung in Teil '"
                                + part + "' in Zeile\n\t" + line + "\tim ged4-Format!");
            }
            String key = equationParts.get(0);
            String value = equationParts.get(1);
            switch (key) {
                case META_DEFAULT_VERTEX_COLOR :
                    meta.setDefaultVertexColor(new GedColor(value));
                    break;
                case META_DEFAULT_EDGE_COLOR :
                    meta.setDefaultEdgeColor(new GedColor(value));
                    break;
                case META_DEFAULT_RADIUS :
                    meta.setDefaultRadius(Integer.parseInt(doubleToInt(value)));
                    break;
                case META_BACKGROUND_COLOR :
                    meta.setBackgroundColor(new GedColor(value));
                    break;
                case META_SHOW_GRID :
                    switch (value) {
                        case "true" :
                            meta.setShowGrid(true);
                            break;
                        case "false" :
                            meta.setShowGrid(false);
                            break;
                         default:
                            throw new WrongSourceFormatException(""
                                    + "Unbekannter Wert '" + value
                                    + "' für showGrid in Teil '" + part
                                    + "' in Zeile\n\t" + line
                                    + "\tim ged4-Format!");
                    }
                    break;
                case META_GRID_DISTANCE :
                    meta.setGridDistance(Integer.parseInt(doubleToInt(value)));
                    break;
                case META_SNAP_IN_GRID :
                    switch (value) {
                        case "true" :
                            meta.setSnapInGrid(true);
                            break;
                        case "false" :
                            meta.setSnapInGrid(false);
                            break;
                         default:
                            throw new WrongSourceFormatException(""
                                    + "Unbekannter Wert '" + value
                                    + "' für setSnapInGrid in Teil '"
                                    + part + "' in Zeile\n\t" + line
                                    + "\tim ged4-Format!");
                    }
                    break;
                case META_GRID_COLOR :
                    meta.setGridColor(new GedColor(value));
                    break;
                case META_BACKGROUND_PICTURE :
                    meta.setBackgroundPicture(value);
                    break;
                case META_DEFAULT_EDGE_WIDTH :
                    meta.setDefaultEdgeWidth(Integer.parseInt(doubleToInt(value)));
                    break;
            }
        }
    }

    private void workOnVertexLine(String line, List<String> parts) {
        Vertex vertex = new Vertex();
        int hasToHaveNumber = -1;
        for (String part : parts) {
            List<String> equationParts = Arrays.asList(part.split("=", -1));
            if (equationParts.size() != 2) {
                throw new WrongSourceFormatException("Nicht "
                        + "erlaubte Anzahl in Gleichung in Teil '" + part
                        + "' in Zeile\n\t" + line + "\tim ged4-Format!");
            }
            String key = equationParts.get(0);
            String value = equationParts.get(1);
            switch (key) {
                case VERTEX_NUMBER :
                    hasToHaveNumber = Integer.parseInt(value);
                    break;
                case VERTEX_X :
                    vertex.setX(Integer.parseInt(doubleToInt(value)));
                    break;
                case VERTEX_Y :
                    vertex.setY(Integer.parseInt(doubleToInt(value)));
                    break;
                case VERTEX_COLOR :
                    vertex.setColor(new GedColor(value));
                    break;
                case VERTEX_RADIUS :
                    vertex.setRadius(Integer.parseInt(doubleToInt(value)));
                    break;
                case VERTEX_LABEL :
                    vertex.setLabel(value);
                    break;
                case VERTEX_LABEL_X_DISTANCE :
                    vertex.getLabelDistance().setX(Integer.parseInt(doubleToInt(value)));
                    break;
                case VERTEX_LABEL_Y_DISTANCE :
                    vertex.getLabelDistance().setY(Integer.parseInt(doubleToInt(value)));
                    break;
                case VERTEX_LABEL_COLOR :
                    vertex.setLabelColor(new GedColor(value));
                    break;
                case VERTEX_WEIGHT :
                    vertex.setWeight(Double.parseDouble(value));
                    break;
                case VERTEX_WEIGHT_X_DISTANCE :
                    vertex.getWeightDistance().setX(Integer.parseInt(doubleToInt(value)));
                    break;
                case VERTEX_WEIGHT_Y_DISTANCE :
                    vertex.getWeightDistance().setY(Integer.parseInt(doubleToInt(value)));
                    break;
                case VERTEX_WEIGHT_COLOR :
                    vertex.setWeightColor(new GedColor(value));
                    break;
            }
        }

        graph.addVertex(vertex);
        if (vertex.getIndex() != hasToHaveNumber) {
            throw new WrongSourceFormatException("Die Ecke\n\t" + vertex
                    + "\that nicht die vorgesehene Nummer " + hasToHaveNumber
                    + " zugewiesen bekommen. In Zeile\n\t" + line + "\tim ged4-Format!");
        }
    }

    private void workOnEdgeLine(String line, List<String> parts) {
        AdjacenceEntry entry = new AdjacenceEntry();
        int hasToHaveStartIndex = -1;
        int hasToHaveEndIndex = -1;
        for (String part : parts) {
            List<String> equationParts = Arrays.asList(part.split("=", -1));
            if (equationParts.size() != 2) {
                throw new WrongSourceFormatException("Nicht "
                        + "erlaubte Anzahl in Gleichung in Teil '" + part
                        + "' in Zeile\n\t" + line + "\tim ged4-Format!");
            }
            String key = equationParts.get(0);
            String value = equationParts.get(1);
            switch (key) {
                case EDGE_START_VERTEX :
                    int startIndex = Integer.parseInt(value);
                    hasToHaveStartIndex = startIndex;
                    break;
                case EDGE_END_VERTEX :
                    int endIndex = Integer.parseInt(value);
                    hasToHaveEndIndex = endIndex;
                    entry.setNeighbourIndex(endIndex);
                    break;
                case EDGE_COLOR :
                    entry.setColor(new GedColor(value));
                    break;
                case EDGE_STYLE :
                    switch (value) {
                        case EDGE_STYLE_NORMAL :
                            entry.setStyle(EdgeStyle.NORMAL);
                            break;
                        case EDGE_STYLE_DASHED :
                            entry.setStyle(EdgeStyle.DASHED);
                            break;
                        case EDGE_STYLE_DOTTED :
                            entry.setStyle(EdgeStyle.DOTTED);
                            break;
                        default:
                            throw new WrongSourceFormatException(""
                                    + "Unbekannter Stil '" + value
                                    + "' in Teil '" + part
                                    + "' in Zeile\n\t" + line
                                    + "\tim ged4-Format!");
                    }
                    break;
                case EDGE_WIDTH :
                    entry.setWidth(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_LABEL :
                    entry.setLabel(value);
                    break;
                case EDGE_LABEL_X_DISTANCE :
                    entry.getLabelDistance().setX(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_LABEL_Y_DISTANCE :
                    entry.getLabelDistance().setY(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_LABEL_COLOR :
                    entry.setLabelColor(new GedColor(value));
                    break;
                case EDGE_WEIGHT :
                    entry.setWeight(Double.parseDouble(value));
                    break;
                case EDGE_WEIGHT_X_DISTANCE :
                    entry.getWeightDistance().setX(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_WEIGHT_Y_DISTANCE :
                    entry.getWeightDistance().setY(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_WEIGHT_COLOR :
                    entry.setWeightColor(new GedColor(value));
                    break;
                case EDGE_CAPACITY :
                    entry.setCapacity(Double.parseDouble(value));
                    break;
                case EDGE_CAPACITY_X_DISTANCE :
                    entry.getCapacityDistance().setX(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_CAPACITY_Y_DISTANCE :
                    entry.getCapacityDistance().setY(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_CAPACITY_COLOR :
                    entry.setCapacityColor(new GedColor(value));
                    break;
                case EDGE_FLOW :
                    entry.setFlow(Double.parseDouble(value));
                    break;
                case EDGE_FLOW_X_DISTANCE :
                    entry.getFlowDistance().setX(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_FLOW_Y_DISTANCE :
                    entry.getFlowDistance().setY(Integer.parseInt(doubleToInt(value)));
                    break;
                case EDGE_FLOW_COLOR :
                    entry.setFlowColor(new GedColor(value));
                    break;
            }
        }
        if (hasToHaveStartIndex == -1) {
            throw new WrongSourceFormatException("Die Kante\n\t"
                    + entry
                    + "\that keinen Startindex. In Zeile\n\t" + line
                    + "\tim ged4-Format!");
        }
        if (hasToHaveEndIndex == -1) {
            throw new WrongSourceFormatException("Die Kante\n\t"
                    + entry
                    + "\that keinen Endindex. In Zeile\n\t" + line
                    + "\tim ged4-Format!");
        }
        graph.addEdge(hasToHaveStartIndex, entry);
    }

    /** Diese Methode ist nur dafür dar, alte Dateien im ged4-Format zu retten... */
    private String doubleToInt(String perhapsNumber) {
        return NumberString.removeZeroEnding(perhapsNumber);
        // Später wird die Methode unnötig sein! Schadet aber auch nicht.
    }


}
