package de.duehl.math.graph.ged.ui.dialogs;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.edge.AdjacenceEntry;
import de.duehl.math.graph.ged.graph.edge.AdjacenceList;
import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.math.graph.ged.ui.elements.SubGraphToggleButton;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.components.MultipleElementsPanel;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;

/**
 * Diese Klasse öffnet einen Dialog zur Auswahl derjenigen Kanten, die unter mehreren getroffenen
 * Kanten gelöscht werden sollen.
 *
 * @version 1.02     2019-05-24
 * @author Christian Dühl
 */

public class EdgeDeleteDialog extends ModalDialogBase {

    private static final int SUB_GRAPH_SIZE = 150;

    /** Graph, der dargestellt wird. */
    private final Graph graph;

    /**
     * Menge mit den Kanten als Map Starteckenindex auf Liste der von dort beginnenden Kanten, die
     * in der Nähe einer bestimmten Bildschirmkoordinate verlaufen.
     */
    private final Map<Integer, AdjacenceList> map;

    /** Koordinaten, an denen der Dialog angezeigt werden soll. */
    private final PixelPoint coords;

    /** Enthält die grafischen Elemente. */
    private final GuiElements elements;

    /** Liste mit den Buttons zum Löschen der Kanten. */
    private final List<SubGraphToggleButton> subGraphButtons;

    /**
     * Menge mit den Kanten als Map Starteckenindex auf Liste der von dort beginnenden Kanten, die
     * in der Nähe einer bestimmten Bildschirmkoordinate verlaufen, das nur noch die Kanten
     * enthält, die entfernt werden sollen.
     */
    private final Map<Integer, AdjacenceList> deleteMap;

    /**
     * Konstruktor
     *
     * @param graph
     *            Zugrundeliegender Graph.
     * @param map
     *            Menge mit den Kanten als Map Starteckenindex auf Liste der von dort beginnenden
     *            Kanten, die in der Nähe einer bestimmten Bildschirmkoordinate verlaufen.
     * @param coords
     *            Koordinaten, an denen der Dialog angezeigt werden soll.
     * @param elements
     *            Enthält die grafischen Elemente
     * @return Ebensolches Map, das nur noch die Kanten enthält, die entfernt werden sollen.
     */
    public EdgeDeleteDialog(Graph graph, Map<Integer, AdjacenceList> map, PixelPoint coords,
            GuiElements elements) {
        super(elements.getFrameLocation(), elements.getProgramImage(),
                "Zu löschende Kanten auswählen:");
        addEscapeBehaviour();

        this.graph = graph;
        this.map = map;
        this.coords = coords;
        this.elements = elements;

        deleteMap = new HashMap<>();
        subGraphButtons = new ArrayList<>();

        fillDialog();
    }

    @Override
    protected void populateDialog() {
        add(createGraphPanel(), BorderLayout.CENTER);
        add(createButtonPanel(), BorderLayout.SOUTH);

        setLocation();
    }

    private void setLocation() {
        MouseClickLocationSetter setter = new MouseClickLocationSetter(this, coords, elements);
        setter.setLocation();
    }

    /**
     * Erzeugt das Panel mit den Teilgraphen, die die einzelnen Ecken darstellen, die zu löschen
     * sind.
     */
    private JComponent createGraphPanel() {
        for (Graph subGraph : createSubgraphs()) {
            SubGraphToggleButton button = new SubGraphToggleButton(subGraph, SUB_GRAPH_SIZE);
            subGraphButtons.add(button);
        }

        MultipleElementsPanel<SubGraphToggleButton> panel =
                new MultipleElementsPanel<>(subGraphButtons, 5, 1);
        return panel;
    }

    /** Bildet aus der map Teilgraphen, die nur diese Kante enthalten. */
    private List<Graph> createSubgraphs() {
        List<Graph> graphs = new ArrayList<>();

        for (int startIndex : map.keySet()) {
            for (AdjacenceEntry entry : map.get(startIndex)) {
                Graph newGraph = graph.createEdgeSubGraph(startIndex, entry, SUB_GRAPH_SIZE);
                graphs.add(newGraph);
            }
        }

        return graphs;
    }

    /** Erzeugt das Panel mit Abbruch- und OK-Button. */
    private JComponent createButtonPanel() {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new BorderLayout());
        GuiTools.createTitle(buttonPanel);

        buttonPanel.add(createQuitButton(), BorderLayout.WEST);
        buttonPanel.add(createOkButton(), BorderLayout.EAST);

        return buttonPanel;
    }

    private JButton createQuitButton() {
        JButton button = new JButton("Abbrechen");
        button.addActionListener(e -> quit());
        return button;
    }

    private JButton createOkButton() {
        JButton button = new JButton("OK");
        button.addActionListener(e -> pressedOkButton());
        return button;
    }

    private void quit() {
        deleteMap.clear();
        setVisible(false);
        dispose();
    }

    /** Reagiert auf den Druck auf den OK-Button. */
    private void pressedOkButton() {
        List<Integer> indicesToDelete = new ArrayList<>();
        for (int index = 0; index < subGraphButtons.size(); ++index) {
            JToggleButton button = subGraphButtons.get(index);
            if (button.isSelected()) {
                indicesToDelete.add(index);
            }
        }

        deleteMap.clear();
        int index = 0;
        for (int startIndex : map.keySet()) {
            for (AdjacenceEntry entry : map.get(startIndex)) {
                if (indicesToDelete.contains(index)) {
                    if (!deleteMap.containsKey(startIndex)) {
                        AdjacenceList newList = new AdjacenceList();
                        deleteMap.put(startIndex, newList);
                    }
                    deleteMap.get(startIndex).add(entry);
                }
                ++index;
            }
        }

        setVisible(false);
        dispose();
    }

    /**
     * Getter für die Menge mit den Kanten als Map Starteckenindex auf Liste der von dort
     * beginnenden Kanten, die in der Nähe einer bestimmten Bildschirmkoordinate verlaufen, das nur
     * noch die Kanten enthält, die entfernt werden sollen.
     */
    public Map<Integer, AdjacenceList> getDeleteMap() {
        return deleteMap;
    }

}
