package de.duehl.swing.ui.dialogs.base;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Window;

import javax.swing.JRootPane;

import de.duehl.basics.logging.Logger;
import de.duehl.swing.logic.LongTimeProcessInformer;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.base.additional.DialogCloser;
import de.duehl.swing.ui.dialogs.base.additional.WindowRefresher;
import de.duehl.swing.ui.elements.DisabledGlassPane;
import de.duehl.swing.ui.handler.error.GlassPaneDeactivatingErrorHandler;
import de.duehl.swing.ui.key.BindKeysOnRootPane;

/**
 * Diese Klasse ist die gemeinsame abstrakte Basis der Dialoge mit JFrame oder JDialog.
 *
 * @version 1.01     2019-05-27
 * @author Christian Dühl
 */

public abstract class AbstractDialogBase implements DialogCloser, LongTimeProcessInformer {

    protected static final Dimension IGNORE_DIMENSION = new Dimension(-1, -1);

    /** Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private final Point parentLocation;

    /** Icon für das Programm. */
    private final Image programImage;

    /** Das Fenster, das dargestellt wird. */
    private Window window; // de fakto final

    /** Größe des Fensters. */
    private final Dimension windowDimension;

    /** Titel des Fensters. */
    protected final String windowTitle;

    /** Farbverwaltung für die Gui. */
    private final Colorizer colorizer;

    /** GlassPane, die die Bedienung der Gui-Elemente verhindert, wenn ein Prozess im Gang ist. */
    private final DisabledGlassPane glassPane;

    /** Gibt an, ob absolut statt relativ zur Position des Parents positioniert werden soll. */
    private boolean hasAbsoluteLocation;

    /** Absolute Position, falls hasAbsoluteLocation true ist. */
    private Point absoluteWindowLocation;

    /**
     * Ist diese Option gesetzt, so wird die Größe des Fensters nicht fest gesetzt, sondern
     * mit pack() auf die kleinste passende Größe festgesetzt.
     *
     * Man kann optional mit setPreferredSize() eine minimal gewünschte Größe angeben.
     */
    private boolean ignoreSize;

    private int minWidth = 0;
    private int minHeight = 0;

    /**
     * Zähler, wird bei jedem startLongTimeProcess erhöht und bei jedem endLongTimeProcess
     * verringert.
     */
    private int longTimeProccessCounter = 0;

    /** Gibt an, ob der longTimeProccessCounter berücksichtigt werden soll. */
    private boolean useLongTimeProccessCounter = false;

    /**
     * Konstruktor für ein gepacktes Fenster ohne fest vorgegebene Dimension.
     *
     * @param programImage
     *            Icon für das Programm.
     * @param windowTitle
     *            Titel des Fensters.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param window
     *            Komponente mit dem JDialog oder JFrame.
     */
    public AbstractDialogBase(Image programImage, String windowTitle, Colorizer colorizer) {
        this(new Point(0, 0), programImage, windowTitle, IGNORE_DIMENSION, colorizer);
        ignoreSize();
    }

    /**
     * Konstruktor.
     *
     * @param programImage
     *            Icon für das Programm.
     * @param windowTitle
     *            Titel des Fensters.
     * @param windowDimension
     *            Größe des Fensters.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param window
     *            Komponente mit dem JDialog oder JFrame.
     */
    public AbstractDialogBase(Image programImage,
            String windowTitle, Dimension windowDimension, Colorizer colorizer) {
        this(new Point(0, 0), programImage, windowTitle, windowDimension, colorizer);
    }

    /**
     * Konstruktor für ein gepacktes Fenster ohne fest vorgegebene Dimension.
     *
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieses Fensters erzeugt wird.
     * @param programImage
     *            Icon für das Programm.
     * @param windowTitle
     *            Titel des Fensters.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param window
     *            Komponente mit dem JDialog oder JFrame.
     */
    public AbstractDialogBase(Point parentLocation, Image programImage,
            String windowTitle, Colorizer colorizer) {
        this(parentLocation, programImage, windowTitle, IGNORE_DIMENSION, colorizer);
        ignoreSize();
    }

    /**
     * Konstruktor.
     *
     * @param parentLocation
     *            Position des Rahmens der Oberfläche, vor der dieses Fensters erzeugt wird.
     * @param programImage
     *            Icon für das Programm.
     * @param windowTitle
     *            Titel des Fensters.
     * @param windowDimension
     *            Größe des Fensters.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param window
     *            Komponente mit dem JDialog oder JFrame.
     */
    public AbstractDialogBase(Point parentLocation, Image programImage, String windowTitle,
            Dimension windowDimension, Colorizer colorizer) {
        this.parentLocation = parentLocation;
        this.programImage = programImage;
        this.windowDimension = windowDimension;
        this.windowTitle = windowTitle;
        this.colorizer = colorizer;

        hasAbsoluteLocation = false;

        ignoreSize = false;
        minWidth = 0;
        minHeight = 0;

        glassPane = new DisabledGlassPane();
    }

    /** Setter für die Komponente mit dem JDialog oder JFrame. */
    protected final void setWindow(Window window) {
        this.window = window;
        glassPane.connectTo(window);
    }

    /** Baut die Gui auf. */
    protected void fillDialog() {
        setColors(window);
        if (null != programImage) {
            window.setIconImage(programImage);
        }
        window.setLayout(new BorderLayout());

        populateDialog();

        if (ignoreSize || windowDimension.equals(IGNORE_DIMENSION)) { // sollte immer beides sein
            window.setMinimumSize(new Dimension(minWidth, minHeight));
            window.pack();
        }
        else {
            window.setPreferredSize(windowDimension);
        }

        window.pack();
        //window.setResizable(false); // TODO Optional per Methodenaufruf ...
        window.setLocation(calculateMyLocation());
    }

    /** Füllt den Rahmen des Dialoges. */
    protected abstract void populateDialog();

    private Point calculateMyLocation() {
        if (hasAbsoluteLocation) {
            return absoluteWindowLocation;
        }
        else {
            double x = parentLocation.getX() + 150;
            double y = parentLocation.getY() + 90;
            Point point = new Point();
            point.setLocation(x, y);
            return point;
        }
    }

    /** Legt die Vordergrundfarbe fest, wenn ein Colorizer hinterlegt wurde, dann aus diesem. */
    protected final Color determineForeground() {
        if (null != colorizer) {
            return colorizer.getForegroundColor();
        }
        else {
            return new Color(0x00, 0x00, 0x00); // new Color(0x00, 0x00, 0x37);
            //return dialog.getForeground(); // NULL !!!
        }
    }

    /** Legt die Hintergrundfarbe fest, wenn ein Colorizer hinterlegt wurde, dann aus diesem. */
    protected final Color determineBackground() {
        if (null != colorizer) {
            return colorizer.getBackgroundColor();
        }
        else {
            return new Color(0xff, 0xff, 0xff);
            //return dialog.getBackground();
        }
    }

    /**
     * Fügt eine Komponente zum Rahmen des Programms hinzu
     *
     * @param component
     *            Komponente.
     * @param constraints
     *            Gibt die Position an (z.B. BorderLayout.CENTER).
     */
    protected final void add(Component component, Object constraints) {
        window.add(component, constraints);
    }

    /** Beendet das Fenster. */
    @Override
    public final void closeDialog() {
        deactivateGlassPane();
        window.setVisible(false);
        /*
         * Gegen gelegentliche Fehler der Art "Exception while removing
         * reference: java.lang.InterruptedException"
         * vgl. http://stackoverflow.com/questions/2873449/
         * occasional-interruptedexception-when-quitting-a-swing-application
         */
        System.gc();
        window.dispose();
    }

    /**
     * Zeigt das Fenster an oder versteckt ihn.
     *
     * @param visible
     *            Gibt an, ob das Fenster angezeigt (true) oder versteckt (false) werden soll.
     */
    public void setVisible(boolean visible) {
        /* Kein SwingUtilities.invokeLater(), sonst stoppt der aufrufende Prozess nicht! */
        window.setVisible(visible);
    }

    /** Erneuert die Anzeige des Dialogs. */
    protected void refresh() {
        window.revalidate();
        window.repaint();
    }

    /** Gibt ein Objekt zurück, das die Anzeige des Dialogs erneuern kann. */
    protected WindowRefresher createRefresher() {
        return () -> refresh();
    }

    /** Getter für die Komponente des Dialogs, der dargestellt wird. */
    protected Component getDialog() {
        return window;
    }

    /** Färbt die übergebene Komponente ein, falls ein Colorizer übergeben wurde. */
    protected void setColors(Component component) {
        if (colorizer != null) {
            colorizer.setColors(component);
        }
    }

    /** Getter für die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird. */
    protected final Point getParentLocation() {
        return parentLocation;
    }

    /** Getter für das Icon für das Programm. */
    public final Image getProgramImage() {
        return programImage;
    }

    /** Getter für den Colorizer. */
    protected final Colorizer getColorizer() {
        return colorizer;
    }

    /** Verlangt nach dem Focus. */
    public void requestFocus() {
        window.requestFocus();
    }

    /**
     * Setzt die bevorzugte Größe des Fensters und überschreibt damit die beim Konstruktor
     * mitgegebene.
     */
    public void setPreferredSize(Dimension dimension) {
        window.setPreferredSize(dimension);
    }

    /**
     * Ignoriert bei fillDialog() die Größenangabe und packt das Fenster oder den Dialog.
     *
     * Eine minimale Größe kann mit setMinimumSize(), oder setMinimumWidth() und setMinimumHeight()
     * angegeben werden.
     */
    protected void ignoreSize() {
        ignoreSize = true;
    }

    /**
     * Setzt die minimale Größe des Fensters.
     *
     * Hierfür muss im Konstruktor ignoreSize() aufgerufen werden, um die absolute Fensterdimension
     * zu ignorieren.
     */
    protected void setMinimumSize(Dimension dimension) {
        window.setMinimumSize(dimension);
        minWidth = (int) dimension.getWidth();
        minHeight = (int) dimension.getHeight();
    }

    /**
     * Setzt die minimale Größe des Fensters.
     *
     * Hierfür muss im Konstruktor ignoreSize() aufgerufen werden, um die absolute Fensterdimension
     * zu ignorieren.
     */
    protected void setMinimumSize(int minWidth, int minHeight) {
        this.minWidth = minWidth;
        this.minHeight = minHeight;
        window.setMinimumSize(new Dimension(minWidth, minHeight));
    }

    /**
     * Setzt die minimale Breite.
     *
     * Hierfür muss im Konstruktor ignoreSize() aufgerufen werden, um die absolute Fensterdimension
     * zu ignorieren.
     */
    protected void setMinimumWidth(int minWidth) {
        this.minWidth = minWidth;
    }

    /**
     * Setzt die minimale Höhe für den Fall, dass bei fillDialog() die Größenangabe ignoriert
     * werden soll.
     */
    protected void setMinimumHeight(int minHeight) {
        this.minHeight = minHeight;
    }

    /**
     * Macht die GlassPane sichtbar und setzt den Mauscursor aus den Wait-Cursor.
     *
     * @param text
     *            Diese Nachricht wird in der Mitte der GlassPane angezeigt, darunter läuft eine
     *            Stoppuhr um anzuzeigen, wie lange die GlassPane nun schon aktiv ist.
     */
    private void activateGlassPane(String message) {
        glassPane.activate(message + " ...");
    }

    /** Versteckt die GlassPane wieder und beendet die Stoppuhr. */
    private void deactivateGlassPane() {
        glassPane.deactivate();
    }

    /** Zum Merken des alten Cursors. */
    private Cursor saveCursor;

    /**
     * Startet einen Prozess, der eine Weile braucht.
     *
     * @param message
     *            Anzuzeigende Nachricht.
     */
    @Override
    public synchronized void startLongTimeProcess(String message) {
        if (useLongTimeProccessCounter) {
            ++longTimeProccessCounter;
        }

        if (!useLongTimeProccessCounter || longTimeProccessCounter == 1) {
            activateGlassPane(message);
            saveCursor = window.getCursor();
            window.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        }
    }

    /** Beendet einen Prozess, der eine Weile braucht. */
    @Override
    public synchronized void endLongTimeProcess() {
        if (useLongTimeProccessCounter) {
            --longTimeProccessCounter;
        }

        if (!useLongTimeProccessCounter || longTimeProccessCounter == 0) {
            deactivateGlassPane();
            window.setCursor(saveCursor);
        }
    }

    /**
     * Damit verschiedene Prozesse (zum Beispiel das Einlesen von unterschiedlichen Listen
     * von Dateien (wie z.B. in der WzPreparationGui), wird mitgezählt, ob auch so oft
     * endLongTimeProzess() aufgerufen wurde, wie vorher startLongTimeProzess(). Erst dann
     * wird die GlassPane wirklich deaktiviert. Die neuen Titel der GlassPane werden
     * allerdings wirklich angezeigt.
     *
     * Standardmäßig schaltet das erste endLongTimeProzess() die GlassPane ab.
     */
    public void useLongTimeProccessCounter() {
        this.useLongTimeProccessCounter = true;
    }

    /** Setzt die Lokation. */
    public void setLocation(int x, int y) {
        setLocation(new Point(x, y));
    }

    protected void setLocation(Point location) {
        hasAbsoluteLocation = true;
        absoluteWindowLocation = location;
        window.setLocation(location);
    }

    /** Gibt die Position des Dialogs an. */
    public Point getLocation() {
        return window.getLocation();
    }

    protected void dispose() {
        window.dispose();
    }

    /** Getter für das Fenster, das dargestellt wird, als Komponente. */
    public Component getWindowAsComponent() {
        return window;
    }

    protected Window getWindow() {
        return window;
    }

    /** Zeigt den Waitcursor an. */
    public void waitcursorOn() {
        GuiTools.waitcursorOn(window);
    }

    /** Zeigt den normalen Cursor an. */
    public void waitcursorOff() {
        GuiTools.waitcursorOff(window);
    }

//    /**
//     * Getter für die GlassPane, die die Bedienung der Gui-Elemente verhindert, wenn ein Prozess im
//     * Gang ist.
//     */
//    protected DisabledGlassPane getGlassPane() {
//        return glassPane;
//    }

    /** Erzeugt einen passenden ErrorHandler, der sich auch um die GalssPane kümmert. */
    public GlassPaneDeactivatingErrorHandler createErrorHandler() {
        GlassPaneDeactivatingErrorHandler error = new GlassPaneDeactivatingErrorHandler(
                programImage, getWindowAsComponent(), glassPane);
        return error;
    }

    public GlassPaneDeactivatingErrorHandler createErrorHandler(Logger logger) {
        GlassPaneDeactivatingErrorHandler error = new GlassPaneDeactivatingErrorHandler(
                programImage, getWindowAsComponent(), glassPane, logger);
        return error;
    }

    /** Repaints this component. */
    protected void repaint() {
        window.repaint();
    }

    /** Validates this container and all of its subcomponents. */
    public void validate() {
        window.validate();
    }

    /** Revalidates the component hierarchy up to the nearest validate root. */
    public void revalidate() {
        window.revalidate();
    }

    /** Gibt die RootPane des Dialogs oder des Frames zurück. */
    protected abstract JRootPane getRootPane();

    /** Fügt die Tastaturbindung Escape hinzu. */
    protected void setKeyBindingEscape(Runnable escapeRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingEscape(escapeRunnable);
        /*
         * Man kann leider keine Klassenvariable bindKeysOnRootPane verwenden, da beim
         * Aufruf des Konstruktors der Dialog bzw. Frame noch nicht existiert und
         * die Keybindings in der Regel vor fillDialog gesetzt werden, weil das das
         * letzte Kommando im Konstruktor der verwendenden Klassen sein soll.
         * Daher muss hier leider immer ein BindKeysOnRootPane-Objekt erzeugt werden.
         *
         * Möglich wäre jeweils initialiseBindKeysOnRootPane() aufzurufen, und falls null dann
         * das Objekt zu erzeugen, aber das ist auch irgendwie hässlich, dann lieber so!
         */
    }

    /** Fügt die Tastaturbindung Home (Pos1) hinzu. */
    protected void setKeyBindingHome(Runnable homeRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingHome(homeRunnable);
    }

    /** Fügt die Tastaturbindung End (Ende) hinzu. */
    protected void setKeyBindingEnd(Runnable endRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingEnd(endRunnable);
    }

    /** Fügt die Tastaturbindung Ctrl-Home (Ctrl-Pos1) hinzu. */
    protected void setKeyBindingCtrlHome(Runnable ctrlHomeRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingCtrlHome(ctrlHomeRunnable);
    }

    /** Fügt die Tastaturbindung Ctrl-End (Ctrl-Ende) hinzu. */
    protected void setKeyBindingCtrlEnd(Runnable ctrlEndRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingCtrlEnd(ctrlEndRunnable);
    }

    /** Fügt die Tastaturbindung PageUp (Bild auf) hinzu. */
    protected void setKeyBindingPageUp(Runnable pageUpRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingPageUp(pageUpRunnable);
    }

    /** Fügt die Tastaturbindung PageDown (Bild runter) hinzu. */
    protected void setKeyBindingPageDown(Runnable pageDownRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingPageDown(pageDownRunnable);
    }

    /** Fügt die Tastaturbindung Enter hinzu. */
    protected void setKeyBindingEnter(Runnable enterRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingEnter(enterRunnable);
    }

    /** Fügt die Tastaturbindung Ctrl-Enter hinzu. */
    protected void setKeyBindingCtrlEnter(Runnable ctrlEnterRunnable) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(getRootPane());
        bind.setKeyBindingCtrlEnter(ctrlEnterRunnable);
    }

}
